// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#include "../../config.h"
#include <strstream.h>

#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <sqlcli1.h>

int main(int argc, char **argv) {

	if (argc<4) {
		cout << "usage: db2test server query iterations" << endl;
		exit(0);
	}

	char	*server=argv[1];
	char	*query=argv[2];
	int	iterations=atoi(argv[3]);
	int	length=strlen(query);
	char	field[4001];

	SQLHENV		env;
	SQLHDBC		dbc;
	SQLHSTMT	stmt;
	SQLSMALLINT	ncols;

	// init the timer
	time_t	starttime=time(NULL);
	clock();

	for (int count=0; count<iterations; count++) {

		SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&env);
		SQLAllocHandle(SQL_HANDLE_DBC,env,&dbc);
		SQLConnect(dbc,(SQLCHAR *)server,SQL_NTS,
				(SQLCHAR *)"",SQL_NTS,
				(SQLCHAR *)"",SQL_NTS);

		// execute the query
		SQLAllocHandle(SQL_HANDLE_STMT,dbc,&stmt);
		SQLPrepare(stmt,(SQLCHAR *)query,length);
		SQLExecute(stmt);
		SQLNumResultCols(stmt,&ncols);

		// run through the rows
		while (SQLFetch(stmt)==SQL_SUCCESS) {

			for (int col=0; col<ncols; col++) {

				// initialize the buffer to an empty string
				field[0]=NULL;

				// get the field
				SQLGetData(stmt,col+1,SQL_C_CHAR,
					(SQLPOINTER)field,4000,NULL);

				cout << "\"" << field << "\",";
			}
			cout << endl;
		}

		// free the result set
		SQLFreeStmt(stmt,SQL_DROP);

		// log off
		SQLDisconnect(dbc);
		SQLFreeHandle(SQL_HANDLE_DBC,dbc);
		SQLFreeHandle(SQL_HANDLE_ENV,env);
	}

	cout << "total system time used: " << clock() << endl;
	cout << "total real time: " << time(NULL)-starttime << endl;
}
