// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#include "../../config.h"
#include <strstream.h>

#include <stdlib.h>
#include <time.h>

extern "C" {
	#include <ctpublic.h>
	#include <ctfunctions.h>
}

CS_CONTEXT	*context;
CS_LOCALE	*locale;
CS_CONNECTION	*conn;
CS_COMMAND	*cmd;
CS_INT		results_type;
CS_INT		ncols;
CS_INT		rows_affected;
CS_DATAFMT	**column;
CS_VOID		**data;
CS_INT		*datalength;
CS_SMALLINT	*nullindicator;
CS_DATAFMT	*convfmt;

int main(int argc, char **argv) {

	if (argc<4) {
		cout << "usage: freetdstest user password server query iterations" << endl;
		exit(0);
	}

	char	*user=argv[1];
	char	*password=argv[2];
	char	*server=argv[3];
	char	*query=argv[4];
	int	iterations=atoi(argv[5]);

	// init the timer
	time_t	starttime=time(NULL);
	clock();

	for (int count=0; count<iterations; count++) {

		context=(CS_CONTEXT *)NULL;

		// define conversion format, will need it alot later
		convfmt=new CS_DATAFMT;
		convfmt->datatype=CS_CHAR_TYPE;
		convfmt->locale=NULL;
		convfmt->format=CS_FMT_NULLTERM;
		convfmt->maxlength=0;
		convfmt->scale=0;
		convfmt->precision=0;


		// allocate a context
		cs_ctx_alloc(CS_VERSION_100,&context);
		// init the context
		ct_init(context,CS_VERSION_100);
		// allocate a connection
		ct_con_alloc(context,&conn);
		// set the user/password to use
		ct_con_props(conn,CS_SET,CS_USERNAME,user,
				CS_NULLTERM,(CS_INT *)NULL);
		ct_con_props(conn,CS_SET,CS_PASSWORD,password,
				CS_NULLTERM,(CS_INT *)NULL);
		// init locale
		locale=NULL;
		cs_loc_alloc(context,&locale);
		cs_locale(context,CS_SET,locale,CS_LC_ALL,(CS_CHAR *)NULL,
				CS_UNUSED,(CS_INT *)NULL);
		// set locale
		ct_con_props(conn,CS_SET,CS_LOC_PROP,(CS_VOID *)locale,
				CS_UNUSED,(CS_INT *)NULL);
		// connect to the database
		ct_connect(conn,server,CS_NULLTERM);
		// allocate a command structure
		ct_cmd_alloc(conn,&cmd);

		// initialize return values
		ncols=0;

		// initiate a language command
		ct_command(cmd,CS_LANG_CMD,query,CS_NULLTERM,CS_UNUSED);

		// send the command
		ct_send(cmd);

		// get the results, sybase is weird, a query can return multiple
		// result sets.  We're only interested in the first one though,
		// the rest will be cancelled
		ct_results(cmd,&results_type);

		ct_res_info(cmd,CS_NUMDATA,(CS_VOID *)&ncols,
				CS_UNUSED,(CS_INT *)NULL);

		rows_affected=CS_NO_COUNT;
		//ct_res_info(cmd,CS_ROW_COUNT,(CS_VOID *)&rows_affected,
				//CS_UNUSED,(CS_INT *)NULL);

		int	length;

		// init a structure to hold the column info
		column=new CS_DATAFMT *[ncols];
		data=new CS_VOID *[ncols];
		datalength=new CS_INT[ncols];
		nullindicator=new CS_SMALLINT[ncols];
	
		for (int i=0; i<ncols; i++) {
	
			column[i]=new CS_DATAFMT;
	
			// init the column
			column[i]->name[0]='\0';
			column[i]->namelen=0;
			column[i]->datatype=0;
			column[i]->format=0;
			column[i]->maxlength=0;
			column[i]->scale=0;
			column[i]->precision=0;
			column[i]->status=0;
			column[i]->count=0;
			column[i]->usertype=0;
			column[i]->locale=NULL;
	
			data[i]=NULL;
			datalength[i]=0;
			nullindicator[i]=0;
		}

		// for each column...
		for (int i=0; i<(int)ncols; i++) {
	
			// get the column description
			ct_describe(cmd,i+1,column[i]);
	
			// append column type to the header
			if (column[i]->datatype==CS_INT_TYPE) {
				column[i]->maxlength = 12;
			} else if (column[i]->datatype==CS_SMALLINT_TYPE) {
				column[i]->maxlength = 7;
			} else if (column[i]->datatype==CS_TINYINT_TYPE) {
				column[i]->maxlength = 4;
			} else if (column[i]->datatype==CS_MONEY_TYPE) {
				column[i]->maxlength = 24;
			} else if (column[i]->datatype==CS_DATETIME_TYPE) {
				column[i]->maxlength = 30;
			} else if (column[i]->datatype==CS_NUMERIC_TYPE) {
				column[i]->maxlength = column[i]->precision + 2;
			} else if (column[i]->datatype==CS_DECIMAL_TYPE) {
				column[i]->maxlength = column[i]->precision + 2;
			} else if (column[i]->datatype==CS_DATETIME4_TYPE) {
				column[i]->maxlength = 30;
			} else if (column[i]->datatype==CS_MONEY4_TYPE) {
				column[i]->maxlength = 24;
			} else if (column[i]->datatype==CS_BINARY_TYPE) {
				column[i]->maxlength *= 2;
			} else if (column[i]->datatype==CS_BIT_TYPE) {
				column[i]->maxlength = 4;
			} else if (column[i]->datatype==CS_REAL_TYPE) {
				column[i]->maxlength = 24;
			} else if (column[i]->datatype==CS_FLOAT_TYPE) {
				column[i]->maxlength = 24;
			} else if (column[i]->datatype==CS_VARBINARY_TYPE) {
				column[i]->maxlength *= 2;
			}
	
			// allocate the data buffer
			data[i]=(CS_VOID *)malloc(column[i]->maxlength+1);

			// bind the column for the fetches
			ct_bind(cmd,i+1,column[i],(CS_VOID *)data[i],
					&datalength[i],&nullindicator[i]);
		}
	
		// go fetch all rows and columns
		CS_INT 		len;
		char		*datastr;
		while (ct_fetch(cmd,CS_UNUSED,CS_UNUSED,CS_UNUSED,NULL)
								==CS_SUCCEED) {

			// send each row back
			for (int col=0; col<(int)ncols; col++) {

				if (!nullindicator[col]) {

					// transform data to a character string
					convfmt->maxlength=datalength[col];
					datastr=new char[datalength[col]+1];
					cs_convert(context,column[col],
							data[col],
							convfmt,
							(CS_VOID *)datastr,
							&len);
					if (datastr) {
						cout << "\"";
						cout << datastr << "\",";
						delete datastr;
					}
				}
			}
			cout << endl;
		}


		// clean up the column stuff
		for (int i=0; i<ncols; i++) {
			delete column[i];
			delete data[i];
		}
		delete column;
		delete data;
		delete datalength;
		delete nullindicator;


		// cancel any extra result sets
		CS_INT	return_code;
		while ((return_code=ct_results(cmd,&results_type))
							==CS_SUCCEED) {
			ct_cancel(conn,cmd,CS_CANCEL_CURRENT);
		}

		ct_cmd_drop(cmd);
		cs_loc_drop(context,locale);
		ct_close(conn,CS_UNUSED);
		ct_con_drop(conn);
		ct_exit(context,CS_UNUSED);
		cs_ctx_drop(context);
		delete convfmt;
	}

	cout << "total system time used: " << clock() << endl;
	cout << "total real time: " << time(NULL)-starttime << endl;
}
