/* Copyright (c) 2000  Adam Kropielnicki
   See the file COPYING for more information */

#include "sql_relay.h"

#include "sqlrclientwrapper.h"

zend_function_entry sql_relay_functions[] = {
	ZEND_FE(sqlrc_alloc,NULL)
	ZEND_FE(sqlrc_setresultsetbuffersize,NULL)
	ZEND_FE(sqlrc_getresultsetbuffersize,NULL)
	ZEND_FE(sqlrc_ping,NULL)
	ZEND_FE(sqlrc_identify,NULL)
	ZEND_FE(sqlrc_sendquery,NULL)
	ZEND_FE(sqlrc_sendfilequery,NULL)
	ZEND_FE(sqlrc_preparequery,NULL)
	ZEND_FE(sqlrc_preparefilequery,NULL)
	ZEND_FE(sqlrc_substitution,NULL)
	ZEND_FE(sqlrc_inputbind,NULL)
	ZEND_FE(sqlrc_defineoutputbind,NULL)
	ZEND_FE(sqlrc_substitutions,NULL)
	ZEND_FE(sqlrc_inputbinds,NULL)
	ZEND_FE(sqlrc_validatebinds,NULL)
	ZEND_FE(sqlrc_executequery,NULL)
	ZEND_FE(sqlrc_getoutputbind,NULL)
	ZEND_FE(sqlrc_cacheon,NULL)
	ZEND_FE(sqlrc_setcachettl,NULL)
	ZEND_FE(sqlrc_cacheoff,NULL)
	ZEND_FE(sqlrc_getcachefilename,NULL)
	ZEND_FE(sqlrc_opencachedresultset,NULL)
	ZEND_FE(sqlrc_opencachedresultsetwithoffset,NULL)
	ZEND_FE(sqlrc_rowcount,NULL)
	ZEND_FE(sqlrc_colcount,NULL)
	ZEND_FE(sqlrc_totalrows,NULL)
	ZEND_FE(sqlrc_affectedrows,NULL)
	ZEND_FE(sqlrc_firstrowindex,NULL)
	ZEND_FE(sqlrc_getnullsasemptystrings,NULL)
	ZEND_FE(sqlrc_getnullsasundefined,NULL)
	ZEND_FE(sqlrc_getfield,NULL)
	ZEND_FE(sqlrc_free,NULL)
	ZEND_FE(sqlrc_endsession,NULL)
	ZEND_FE(sqlrc_suspendsession,NULL)
	ZEND_FE(sqlrc_resumesession,NULL)
	ZEND_FE(sqlrc_resumecachedsession,NULL)
	ZEND_FE(sqlrc_getconnectionport,NULL)
	ZEND_FE(sqlrc_getconnectionsocket,NULL)
	ZEND_FE(sqlrc_getcachefileoffset,NULL)
	ZEND_FE(sqlrc_getcolumnname,NULL)
	ZEND_FE(sqlrc_getrow,NULL)
	ZEND_FE(sqlrc_getrowassoc,NULL)
	ZEND_FE(sqlrc_getcolumnnames,NULL)
	ZEND_FE(sqlrc_getcolumntype,NULL)
	ZEND_FE(sqlrc_getcolumnlength,NULL)
	ZEND_FE(sqlrc_getlongest,NULL)
	ZEND_FE(sqlrc_debugon,NULL)
	ZEND_FE(sqlrc_debugoff,NULL)
	ZEND_FE(sqlrc_getdebug,NULL)
	ZEND_FE(sqlrc_errormessage,NULL)
	{NULL,NULL,NULL}
};


zend_module_entry sql_relay_module_entry = {
	"sql_relay",
	sql_relay_functions,
	NULL,NULL,NULL,NULL,NULL,
	STANDARD_MODULE_PROPERTIES
};


#if COMPILE_DL
DLEXPORT ZEND_GET_MODULE(sql_relay)
#endif

DLEXPORT ZEND_FUNCTION(sqlrc_alloc) {
	zval **server,**port,**socket,**user,**password,**retrytime,**tries;
	sqlrc s;
	if (ZEND_NUM_ARGS() != 7 || 
		zend_get_parameters_ex(7,&server,&port,&socket,
			&user,&password,&retrytime,&tries ) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_string_ex(server);
	convert_to_long_ex(port);
	convert_to_string_ex(socket);
	convert_to_string_ex(user);
	convert_to_string_ex(password);
	convert_to_long_ex(retrytime);
	convert_to_long_ex(tries);
	s=sqlrc_alloc((*server)->value.str.val,
			(*port)->value.lval,(*socket)->value.str.val,
			(*user)->value.str.val,(*password)->value.str.val,
			(*retrytime)->value.lval,(*tries)->value.lval);
	sqlrc_copyReferences(s);
	sqlrc_debugPrintFunction(s,zend_printf);
	RETURN_LONG(s);
}

DLEXPORT ZEND_FUNCTION(sqlrc_setresultsetbuffersize) {
	zval **sqlrcref,**rows;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&rows) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(rows);
	sqlrc_setResultSetBufferSize((*sqlrcref)->value.lval,
					(*rows)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getresultsetbuffersize) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getResultSetBufferSize((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_ping) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_ping((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_identify) {
	zval **sqlrcref;
	char *r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_identify((*sqlrcref)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_cacheon) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_cacheOn((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_setcachettl) {
	zval **sqlrcref,**ttl;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&ttl) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(ttl);
	sqlrc_setResultSetBufferSize((*sqlrcref)->value.lval,
					(*ttl)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_cacheoff) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_cacheOff((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_sendquery) {
	zval **sqlrcref,**query;
	int r;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&query) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(query);
	r=sqlrc_sendQuery((*sqlrcref)->value.lval,(*query)->value.str.val);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_sendfilequery) {
	zval **sqlrcref,**path,**filename;
	int r;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&path,&filename) 
					== FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(path);
	convert_to_string_ex(filename);
	r=sqlrc_sendFileQuery((*sqlrcref)->value.lval,
			(*path)->value.str.val,(*filename)->value.str.val);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_preparequery) {
	zval **sqlrcref,**query;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&query) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(query);
	sqlrc_prepareQuery((*sqlrcref)->value.lval,(*query)->value.str.val);
}

DLEXPORT ZEND_FUNCTION(sqlrc_preparefilequery) {
	zval **sqlrcref,**path,**filename;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&path,&filename) 
					== FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(path);
	convert_to_string_ex(filename);
	sqlrc_prepareFileQuery((*sqlrcref)->value.lval,
			(*path)->value.str.val,(*filename)->value.str.val);
}

DLEXPORT ZEND_FUNCTION(sqlrc_substitution) {
	zval **sqlrcref,**variable,**value,**precision,**scale;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&variable,&value) 
					== FAILURE) {
		if (ZEND_NUM_ARGS() != 5 || 
			zend_get_parameters_ex(5,&sqlrcref,
				&variable,&value,&precision,&scale)== FAILURE) {
			WRONG_PARAM_COUNT;
		}
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(variable);
	if (Z_TYPE_PP(value)==IS_STRING) {
		convert_to_string_ex(value);
		sqlrc_subString((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.str.val);
	} else if (Z_TYPE_PP(value)==IS_LONG) {
		convert_to_long_ex(value);
		sqlrc_subLong((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.lval);
	} else if (ZEND_NUM_ARGS()==4 && Z_TYPE_PP(value)==IS_DOUBLE) {
		convert_to_double_ex(value);
		sqlrc_subDouble((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.dval,
			(*precision)->value.lval,(*scale)->value.lval);
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_inputbind) {
	zval **sqlrcref,**variable,**value,**precision,**scale;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&variable,&value) 
					== FAILURE) {
		if (ZEND_NUM_ARGS() != 5 || 
			zend_get_parameters_ex(5,&sqlrcref,
				&variable,&value,&precision,&scale)== FAILURE) {
			WRONG_PARAM_COUNT;
		}
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(variable);
	if (Z_TYPE_PP(value)==IS_STRING) {
		convert_to_string_ex(value);
		sqlrc_inputBindString((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.str.val);
	} else if (Z_TYPE_PP(value)==IS_LONG) {
		convert_to_long_ex(value);
		sqlrc_inputBindLong((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.lval);
	} else if (ZEND_NUM_ARGS()==4 && Z_TYPE_PP(value)==IS_DOUBLE) {
		convert_to_double_ex(value);
		sqlrc_inputBindDouble((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*value)->value.dval,
			(*precision)->value.lval,(*scale)->value.lval);
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_defineoutputbind) {
	zval **sqlrcref,**variable,**length;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&variable,&length) 
					== FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(variable);
	convert_to_long_ex(length);
	sqlrc_defineOutputBind((*sqlrcref)->value.lval,
			(*variable)->value.str.val,(*length)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_substitutions) {
	zval **sqlrcref,**variables,**values,**precisions,**scales,**var,**val,**precision,**scale;
	int i;
	if (ZEND_NUM_ARGS() == 3 && !
		zend_get_parameters_ex(3,&sqlrcref,&variables,&values) 
					== FAILURE) {
		if (ZEND_NUM_ARGS() != 5 || 
			zend_get_parameters_ex(5,&sqlrcref,
				&variables,&values,&precisions,&scales)== FAILURE) {
			WRONG_PARAM_COUNT;
		} else {
			convert_to_array_ex(precisions);
			convert_to_array_ex(scales);
		}
	}
	convert_to_long_ex(sqlrcref);
	convert_to_array_ex(variables);
	convert_to_array_ex(values);
	for (i=0; i<(*variables)->value.ht->nNumOfElements; i++) {
		zend_hash_index_find((*variables)->value.ht,i,(void *)var);
		zend_hash_index_find((*values)->value.ht,i,(void *)val);
		if (Z_TYPE_PP(val)==IS_STRING) {
			convert_to_string_ex(val);
			sqlrc_subString((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.str.val);
		} else if (Z_TYPE_PP(val)==IS_LONG) {
			convert_to_long_ex(val);
			sqlrc_subLong((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.lval);
		} else if (ZEND_NUM_ARGS()==4 && Z_TYPE_PP(val)==IS_DOUBLE) {
			zend_hash_index_find((*precisions)->value.ht,i,
							(void *)precision);
			zend_hash_index_find((*scales)->value.ht,i,
							(void *)scale);
			convert_to_double_ex(val);
			convert_to_long_ex(precision);
			convert_to_long_ex(scale);
			sqlrc_subDouble((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.dval,
				(*precision)->value.lval,(*scale)->value.lval);
		}
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_inputbinds) {
	zval **sqlrcref,**variables,**values,**precisions,**scales,**var,**val,**precision,**scale;
	int i;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&variables,&values) 
					== FAILURE) {
		if (ZEND_NUM_ARGS() != 5 || 
			zend_get_parameters_ex(5,&sqlrcref,&variables,&values,
						&precisions,&scales)== FAILURE) {
			WRONG_PARAM_COUNT;
		} else {
			convert_to_array_ex(precisions);
			convert_to_array_ex(scales);
		}
	}
	convert_to_long_ex(sqlrcref);
	convert_to_array_ex(variables);
	convert_to_array_ex(values);
	for (i=0; i<zend_hash_num_elements((*variables)->value.ht); i++) {
		zend_hash_index_find((*variables)->value.ht,i,(void *)var);
		zend_hash_index_find((*values)->value.ht,i,(void *)val);
		if (Z_TYPE_PP(val)==IS_STRING) {
			convert_to_string_ex(val);
			sqlrc_inputBindString((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.str.val);
		} else if (Z_TYPE_PP(val)==IS_LONG) {
			convert_to_long_ex(val);
			sqlrc_inputBindLong((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.lval);
		} else if (Z_TYPE_PP(val)==IS_DOUBLE) {
			zend_hash_index_find((*precisions)->value.ht,i,
							(void *)precision);
			zend_hash_index_find((*scales)->value.ht,i,
							(void *)scale);
			convert_to_long_ex(precision);
			convert_to_long_ex(scale);
			convert_to_double_ex(val);
			sqlrc_inputBindDouble((*sqlrcref)->value.lval,
				(*var)->value.str.val,(*val)->value.dval,
				(*precision)->value.lval,(*scale)->value.lval);
		}
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_validatebinds) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_validateBinds((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_executequery) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_executeQuery((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getoutputbind) {
	zval **sqlrcref,**variable;
	char *r;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&variable) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(variable);
	r=sqlrc_getOutputBind((*sqlrcref)->value.lval,
				(*variable)->value.str.val);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcachefilename) {
	zval **sqlrcref;
	char *r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getCacheFileName((*sqlrcref)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_opencachedresultset) {
	zval **sqlrcref,**filename;
	int r;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&filename) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(filename);
	r=sqlrc_openCachedResultSet((*sqlrcref)->value.lval,
					(*filename)->value.str.val);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_opencachedresultsetwithoffset) {
	zval **sqlrcref,**filename,**offset,**row;
	int r;
	if (ZEND_NUM_ARGS() != 4 || 
		zend_get_parameters_ex(4,&sqlrcref,&filename,&offset,&row) 
					== FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_string_ex(filename);
	convert_to_long_ex(offset);
	convert_to_long_ex(row);
	r=sqlrc_openCachedResultSetWithOffset((*sqlrcref)->value.lval,
			(*filename)->value.str.val,(*offset)->value.lval,
			(*row)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_rowcount) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_rowCount((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_colcount) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_colCount((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_totalrows) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_totalRows((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_affectedrows) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_affectedRows((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_firstrowindex) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_firstRowIndex((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getnullsasemptystrings) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_getNullsAsEmptyStrings((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getnullsasundefined) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_getNullsAsNulls((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getfield) {
	zval **sqlrcref,**row,**col;
	char *r="";
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&row,&col) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(row);
	if (Z_TYPE_PP(col)==IS_LONG) {
		convert_to_long_ex(col);
		r=sqlrc_getFieldByIndex((*sqlrcref)->value.lval,
				(*row)->value.lval,(*col)->value.lval);
	} else if (Z_TYPE_PP(col)==IS_STRING) {
		convert_to_string_ex(col);
		r=sqlrc_getFieldByName((*sqlrcref)->value.lval,
				(*row)->value.lval,(*col)->value.str.val);
	}
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_free) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_free((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_endsession) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_endSession((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_suspendsession) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_suspendSession((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_resumesession) {
	zval **sqlrcref,**port,**socket;
	int r;
	if (ZEND_NUM_ARGS() != 3 || 
		zend_get_parameters_ex(3,&sqlrcref,&port,&socket) 
					== FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(port);
	convert_to_string_ex(socket);
	r=sqlrc_resumeSession((*sqlrcref)->value.lval,
				(*port)->value.lval,(*socket)->value.str.val);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_resumecachedsession) {
	zval **sqlrcref,**port,**socket,**cachefilename;
	if (ZEND_NUM_ARGS() != 4 || 
		zend_get_parameters_ex(4,&sqlrcref,
			&port,&socket,&cachefilename) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(port);
	convert_to_string_ex(socket);
	convert_to_string_ex(cachefilename);
	sqlrc_resumeCachedSession((*sqlrcref)->value.lval,
				(*port)->value.lval,(*socket)->value.str.val,
				(*cachefilename)->value.str.val);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getconnectionport) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getConnectionPort((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getconnectionsocket) {
	zval **sqlrcref;
	char *r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getConnectionSocket((*sqlrcref)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcachefileoffset) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getCacheFileOffset((*sqlrcref)->value.lval);
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcolumnname) {
	zval **sqlrcref,**col;
	char *r;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&col) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(col);
	r=sqlrc_getColumnName((*sqlrcref)->value.lval,(*col)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getrow) {
	zval **sqlrcref,**row;
	char **r;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&row) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(row);
	r=sqlrc_getRow((*sqlrcref)->value.lval,(*row)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	if (array_init(return_value) == FAILURE) {
		RETURN_FALSE;
	}
	while (*r) {
		add_next_index_string(return_value,*r,1);
		r++;
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_getrowassoc) {
	zval **sqlrcref,**row;
	char **r,**rC;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&row) == FAILURE) {
		WRONG_PARAM_COUNT;
	}

	convert_to_long_ex(sqlrcref);
	convert_to_long_ex(row);

	rC=sqlrc_getColumnNames((*sqlrcref)->value.lval);
	if (!rC) {
		RETURN_FALSE;
	}

	r=sqlrc_getRow((*sqlrcref)->value.lval,(*row)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	if (array_init(return_value) == FAILURE) {
		RETURN_FALSE;
	}
	while (*r&& *rC) {
		add_assoc_string(return_value,*rC,*r,1);
		r++;
		rC++;
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcolumnnames) {
	zval **sqlrcref;
	char **r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getColumnNames((*sqlrcref)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	if (array_init(return_value) == FAILURE) {
		RETURN_FALSE;
	}
	while (*r) {
		add_next_index_string(return_value,*r,1);
		r++;
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcolumntype) {
	zval **sqlrcref,**col;
	char *r="";
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&col) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	if (Z_TYPE_PP(col)==IS_LONG) {
		convert_to_long_ex(col);
		r=sqlrc_getColumnTypeByIndex((*sqlrcref)->value.lval,
						(*col)->value.lval);
	} else if (Z_TYPE_PP(col)==IS_STRING) {
		convert_to_string_ex(col);
		r=sqlrc_getColumnTypeByName((*sqlrcref)->value.lval,
						(*col)->value.str.val);
	}
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getcolumnlength) {
	zval **sqlrcref,**col;
	int r=0;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&col) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	if (Z_TYPE_PP(col)==IS_LONG) {
		convert_to_long_ex(col);
		r=sqlrc_getColumnLengthByIndex((*sqlrcref)->value.lval,
						(*col)->value.lval);
	} else if (Z_TYPE_PP(col)==IS_STRING) {
		convert_to_string_ex(col);
		r=sqlrc_getColumnLengthByName((*sqlrcref)->value.lval,
						(*col)->value.str.val);
	}
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getlongest) {
	zval **sqlrcref,**col;
	int r=0;
	if (ZEND_NUM_ARGS() != 2 || 
		zend_get_parameters_ex(2,&sqlrcref,&col) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	if (Z_TYPE_PP(col)==IS_LONG) {
		convert_to_long_ex(col);
		r=sqlrc_getLongestByIndex((*sqlrcref)->value.lval,
						(*col)->value.lval);
	} else if (Z_TYPE_PP(col)==IS_STRING) {
		convert_to_string_ex(col);
		r=sqlrc_getLongestByName((*sqlrcref)->value.lval,
						(*col)->value.str.val);
	}
	RETURN_LONG(r);
}

DLEXPORT ZEND_FUNCTION(sqlrc_debugon) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_debugOn((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_debugoff) {
	zval **sqlrcref;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	sqlrc_debugOff((*sqlrcref)->value.lval);
}

DLEXPORT ZEND_FUNCTION(sqlrc_getdebug) {
	zval **sqlrcref;
	int r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_getDebug((*sqlrcref)->value.lval);
	if (r) {
		RETURN_TRUE;
	} else {
		RETURN_FALSE;
	}
}

DLEXPORT ZEND_FUNCTION(sqlrc_errormessage) {
	zval **sqlrcref;
	char *r;
	if (ZEND_NUM_ARGS() != 1 || 
		zend_get_parameters_ex(1,&sqlrcref) == FAILURE) {
		WRONG_PARAM_COUNT;
	}
	convert_to_long_ex(sqlrcref);
	r=sqlrc_errorMessage((*sqlrcref)->value.lval);
	if (!r) {
		RETURN_FALSE;
	}
	RETURN_STRING(r,1);
}
