/* Copyright (c) 2000 Roman Milner
   See the file COPYING for more information */

#include <Python.h>
#include <sqlrclientwrapper.h>

static PyObject *
connect(self, args)
     PyObject *self;
     PyObject *args;
{
  sqlrc cur_connection;
  char *host;
  char *user;
  char *password;
  char *socket;
  int port;
  int retrytime;
  int tries;
  
  if (!PyArg_ParseTuple(args, "sisssii", &host, &port, &socket, &user, &password, &retrytime, &tries))
    return NULL;
  cur_connection = sqlrc_alloc(host, port, socket, user, password, 1, 1);
  sqlrc_copyReferences(cur_connection);
  return Py_BuildValue("i", cur_connection);
}

static PyObject *
sqlr_free(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_free(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
endSession(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_endSession(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
suspendSession(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_suspendSession(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
resumeSession(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int port;
  char *socket;
  int rc;
  if (!PyArg_ParseTuple(args, "lis", &conObj, &port, &socket))
    return NULL;
  rc=sqlrc_resumeSession(conObj,port,socket);
  return Py_BuildValue("i", rc);
}

static PyObject *
resumeCachedSession(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int port;
  char *socket;
  char *cachefilename;
  int rc;
  if (!PyArg_ParseTuple(args, "liss", &conObj, &port, &socket, &cachefilename))
    return NULL;
  rc=sqlrc_resumeCachedSession(conObj,port,socket,cachefilename);
  return Py_BuildValue("i", rc);
}

static PyObject *
getConnectionPort(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc=sqlrc_getConnectionPort(conObj);
  return Py_BuildValue("i",rc);
}

static PyObject *
getConnectionSocket(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char *rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc=sqlrc_getConnectionSocket(conObj);
  return Py_BuildValue("s",rc);
}

static PyObject *
getCacheFileOffset(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc=sqlrc_getCacheFileOffset(conObj);
  return Py_BuildValue("i",rc);
}

static PyObject *
setResultSetBufferSize(self, args)
     PyObject *self;
     PyObject *args;
{
  int rows;
  long conObj;
  if (!PyArg_ParseTuple(args, "li", &conObj, &rows))
    return NULL;
  sqlrc_setResultSetBufferSize(conObj, rows);
  return Py_BuildValue("i", 0);
}

static PyObject *
getResultSetBufferSize(self, args)
     PyObject *self;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc=sqlrc_getResultSetBufferSize(conObj);
  return Py_BuildValue("i", rc);
}


static PyObject *
ping(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_ping(conObj);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("i", rc);
}


static PyObject *
identify(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char *rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_identify(conObj);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("s", strdup(rc));
}

static PyObject *
cacheOn(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_cacheOn(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
setCacheTtl(self, args)
     PyObject *self;
     PyObject *args;
{
  int ttl;
  long conObj;
  if (!PyArg_ParseTuple(args, "li", &conObj, &ttl))
    return NULL;
  sqlrc_setResultSetBufferSize(conObj, ttl);
  return Py_BuildValue("i", 0);
}

static PyObject *
cacheOff(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_cacheOff(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
sendQuery(self, args)
     PyObject *self;
     PyObject *args;
{
  char *sqlString;
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &sqlString))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_sendQuery(conObj, sqlString);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("i", rc);
}

static PyObject *
sendFileQuery(self, args)
     PyObject *self;
     PyObject *args;
{
  char *path;
  char *file;
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "lss", &conObj, &path, &file))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_sendFileQuery(conObj, path, file);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("i", rc);
}

static PyObject *
prepareQuery(self, args)
     PyObject *self;
     PyObject *args;
{
  char *sqlString;
  long conObj;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &sqlString))
    return NULL;
  sqlrc_prepareQuery(conObj, sqlString);
  return Py_BuildValue("i", 0);
}

static PyObject *
prepareFileQuery(self, args)
     PyObject *self;
     PyObject *args;
{
  char *path;
  char *file;
  long conObj;
  if (!PyArg_ParseTuple(args, "lss", &conObj, &path, &file))
    return NULL;
  sqlrc_prepareFileQuery(conObj, path, file);
  return Py_BuildValue("i", 0);
}

static PyObject *
subNone(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  long conObj;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &variable))
    return NULL;
  sqlrc_subString(conObj, variable, NULL);
  return Py_BuildValue("i", 0);
}

static PyObject *
subString(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  char *value;
  long conObj;
  if (!PyArg_ParseTuple(args, "lss", &conObj, &variable, &value))
    return NULL;
  sqlrc_subString(conObj, variable, value);
  return Py_BuildValue("i", 0);
}

static PyObject *
subLong(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  long value;
  long conObj;
  if (!PyArg_ParseTuple(args, "lsl", &conObj, &variable, &value))
    return NULL;
  sqlrc_subLong(conObj, variable, value);
  return Py_BuildValue("i", 0);
}

static PyObject *
subDouble(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  double value;
  short precision;
  short scale;
  long conObj;
  if (!PyArg_ParseTuple(args, "lsdii", &conObj, &variable, &value, &precision, &scale))
    return NULL;
  sqlrc_subDouble(conObj, variable, value, precision, scale);
  return Py_BuildValue("i", 0);
}

static PyObject *
inputBindNone(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  long conObj;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &variable))
    return NULL;
  sqlrc_inputBindString(conObj, variable, NULL);
  return Py_BuildValue("i", 0);
}

static PyObject *
inputBindString(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  char *value;
  long conObj;
  if (!PyArg_ParseTuple(args, "lss", &conObj, &variable, &value))
    return NULL;
  sqlrc_inputBindString(conObj, variable, value);
  return Py_BuildValue("i", 0);
}

static PyObject *
inputBindLong(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  long value;
  long conObj;
  if (!PyArg_ParseTuple(args, "lsl", &conObj, &variable, &value))
    return NULL;
  sqlrc_inputBindLong(conObj, variable, value);
  return Py_BuildValue("i", 0);
}

static PyObject *
inputBindDouble(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  double value;
  short precision;
  short scale;
  long conObj;
  if (!PyArg_ParseTuple(args, "lsdii", &conObj, &variable, &value, &precision, &scale))
    return NULL;
  sqlrc_inputBindDouble(conObj, variable, value, precision, scale);
  return Py_BuildValue("i", 0);
}

static PyObject *
validateBinds(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_validateBinds(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
executeQuery(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_executeQuery(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
defineOutputBind(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  int length;
  long conObj;
  if (!PyArg_ParseTuple(args, "lsi", &conObj, &variable, &length))
    return NULL;
  sqlrc_defineOutputBind(conObj, variable, length);
  return Py_BuildValue("i", 0);
}

static PyObject *
getOutputBind(self, args)
     PyObject *self;
     PyObject *args;
{
  char *variable;
  long conObj;
  char *rc;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &variable))
    return NULL;
  rc=sqlrc_getOutputBind(conObj, variable);
  return Py_BuildValue("s",rc);
}

static PyObject *
getCacheFileName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char  *rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_getCacheFileName(conObj);
  return Py_BuildValue("s", rc);
}

static PyObject *
openCachedResultSet(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  char *filename;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &filename))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_openCachedResultSet(conObj, filename);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("i", rc);
}

static PyObject *
openCachedResultSetWithOffset(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char *filename;
  int offset;
  int row;
  int rc;
  if (!PyArg_ParseTuple(args, "lsii", &conObj, &filename, &offset, &row))
    return NULL;
  Py_BEGIN_ALLOW_THREADS
  rc = sqlrc_openCachedResultSetWithOffset(conObj, filename, offset, row);
  Py_END_ALLOW_THREADS
  return Py_BuildValue("i", rc);
}

static PyObject *
rowCount(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_rowCount(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
colCount(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_colCount(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
totalRows(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_totalRows(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
affectedRows(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_affectedRows(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
firstRowIndex(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  rc = sqlrc_firstRowIndex(conObj);
  return Py_BuildValue("i", rc);
}

static PyObject *
getNullsAsEmptyStrings(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_getNullsAsEmptyStrings(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
getNullsAsNone(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_getNullsAsNulls(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
getFieldByIndex(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char  *rc;
  int row;
  int col;
  if (!PyArg_ParseTuple(args, "lii", &conObj, &row, &col))
    return NULL;
  rc = sqlrc_getFieldByIndex(conObj, row, col);
  return Py_BuildValue("s", rc);
}

static PyObject *
getFieldByName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char  *rc;
  int row;
  char *col;
  if (!PyArg_ParseTuple(args, "lis", &conObj, &row, &col))
    return NULL;
  rc = sqlrc_getFieldByName(conObj, row, col);
  return Py_BuildValue("s", rc);
}

static PyObject *
getColumnTypeByIndex(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char *rc;
  int col;
  if (!PyArg_ParseTuple(args, "li", &conObj, &col))
    return NULL;
  rc = sqlrc_getColumnTypeByIndex(conObj, col);
  return Py_BuildValue("s", rc);
}

static PyObject *
getColumnTypeByName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char *rc;
  char *col;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &col))
    return NULL;
  rc = sqlrc_getColumnTypeByName(conObj, col);
  return Py_BuildValue("s", rc);
}

static PyObject *
getColumnLengthByIndex(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  int col;
  if (!PyArg_ParseTuple(args, "li", &conObj, &col))
    return NULL;
  rc = sqlrc_getColumnLengthByIndex(conObj, col);
  return Py_BuildValue("i", rc);
}

static PyObject *
getColumnLengthByName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  char *col;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &col))
    return NULL;
  rc = sqlrc_getColumnLengthByName(conObj, col);
  return Py_BuildValue("i", rc);
}

static PyObject *
getLongestByIndex(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  int col;
  if (!PyArg_ParseTuple(args, "li", &conObj, &col))
    return NULL;
  rc = sqlrc_getLongestByIndex(conObj, col);
  return Py_BuildValue("i", rc);
}

static PyObject *
getLongestByName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int rc;
  char *col;
  if (!PyArg_ParseTuple(args, "ls", &conObj, &col))
    return NULL;
  rc = sqlrc_getLongestByName(conObj, col);
  return Py_BuildValue("i", rc);
}


static PyObject *
getColumnName(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  char  *rc;
  int col;
  if (!PyArg_ParseTuple(args, "li", &conObj, &col))
    return NULL;
  rc = sqlrc_getColumnName(conObj, col);
  return Py_BuildValue("s", rc);
}

static PyObject *
getColumnNames(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int counter;
  int num_cols;
  char **rc;
  PyObject *my_list;
  my_list = PyList_New(0);
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  num_cols = sqlrc_colCount(conObj);
  rc = sqlrc_getColumnNames(conObj);
  for (counter = 0; counter < num_cols; ++counter) {
    PyList_Append(my_list, Py_BuildValue("s", rc[counter]));
  }
  return my_list;
}

PyObject *
_get_row(sqlrc conObj, int row)
{
  int num_cols;
  int counter;
  char **row_data;
  PyObject *my_list;
  num_cols = sqlrc_colCount(conObj);
  my_list =  PyList_New(num_cols);
  row_data = sqlrc_getRow(conObj, row);
  for (counter = 0; counter < num_cols; ++counter) {
    if (!row_data[counter]) {
        PyList_SetItem(my_list, counter, Py_None);
    } else if (isNumberTypeChar(sqlrc_getColumnTypeByIndex(conObj,counter))) {
      if (!strchr(row_data[counter], '.')) {
          PyList_SetItem(my_list, counter, Py_BuildValue("i", atoi(row_data[counter])));
      } else {
          PyList_SetItem(my_list, counter, Py_BuildValue("f", atof(row_data[counter])));
      }
    } else {
      PyList_SetItem(my_list, counter, Py_BuildValue("s", row_data[counter]));
    }
  }
  return my_list;
}

static PyObject *
getRow(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int row;
  if (!PyArg_ParseTuple(args, "li", &conObj, &row))
    return NULL;
  return _get_row(conObj, row);
}

static PyObject *
getRowDictionary(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  int row;
  PyObject *my_dictionary;
  int counter;
  char *field;
  char *name;
  if (!PyArg_ParseTuple(args, "li", &conObj, &row))
    return NULL;
  my_dictionary=PyDict_New();
  for (counter=0; counter<sqlrc_colCount(conObj); counter++) {
    field=sqlrc_getFieldByIndex(conObj,row,counter);
    name=sqlrc_getColumnName(conObj,counter);
    if (isNumberTypeChar(sqlrc_getColumnTypeByIndex(conObj, counter))) {
      if (!strchr(field,'.')) {
        PyDict_SetItem(my_dictionary,Py_BuildValue("s",name),Py_BuildValue("i",atoi(field)));
      } else {
        PyDict_SetItem(my_dictionary,Py_BuildValue("s",name),Py_BuildValue("f",atof(field)));
      }
    } else {
      PyDict_SetItem(my_dictionary,Py_BuildValue("s",name),Py_BuildValue("s",field));
    }
  }
  return my_dictionary;
}

static PyObject *
getRowRange(self, args)
     PyObject *self;
     PyObject *args;
{
  int beg_row;
  int end_row;
  long conObj;
  int counter;
  PyObject *cur_row;
  PyObject *my_list;
  my_list = PyList_New(0);
  if (!PyArg_ParseTuple(args, "lii", &conObj, &beg_row, &end_row))
    return NULL;
  for (counter = beg_row; counter <= end_row; ++counter) {
    PyList_Append(my_list, _get_row(conObj, counter));
  }
  return my_list;
}

static PyObject *
errorMessage(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  return Py_BuildValue("s", sqlrc_errorMessage(conObj));
}

static PyObject *
debugOn(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_debugOn(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
debugOff(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  sqlrc_debugOff(conObj);
  return Py_BuildValue("i", 0);
}

static PyObject *
getDebug(self, args)
     PyObject *self;
     PyObject *args;
{
  long conObj;
  if (!PyArg_ParseTuple(args, "l", &conObj))
    return NULL;
  return Py_BuildValue("i", sqlrc_getDebug(conObj));
}

static PyMethodDef SQLRMethods[] = {
  {"connect",  connect, METH_VARARGS},
  {"setResultSetBufferSize", setResultSetBufferSize, METH_VARARGS},
  {"getResultSetBufferSize", getResultSetBufferSize, METH_VARARGS},
  {"ping", ping, METH_VARARGS},
  {"identify", identify, METH_VARARGS},
  {"sendQuery", sendQuery, METH_VARARGS},
  {"sendFileQuery", sendFileQuery, METH_VARARGS},
  {"prepareQuery", prepareQuery, METH_VARARGS},
  {"prepareFileQuery", prepareFileQuery, METH_VARARGS},
  {"subNone", subNone, METH_VARARGS},
  {"subString", subString, METH_VARARGS},
  {"subLong", subLong, METH_VARARGS},
  {"subDouble", subDouble, METH_VARARGS},
  {"inputBindNone", inputBindNone, METH_VARARGS},
  {"inputBindString", inputBindString, METH_VARARGS},
  {"inputBindLong", inputBindLong, METH_VARARGS},
  {"inputBindDouble", inputBindDouble, METH_VARARGS},
  {"defineOutputBind", defineOutputBind, METH_VARARGS},
  {"validateBinds", validateBinds, METH_VARARGS},
  {"executeQuery", executeQuery, METH_VARARGS},
  {"getOutputBind", getOutputBind, METH_VARARGS},
  {"cacheOn", cacheOn, METH_VARARGS},
  {"setCacheTtl", setCacheTtl, METH_VARARGS},
  {"getCacheFileName", getCacheFileName, METH_VARARGS},
  {"cacheOff", cacheOff, METH_VARARGS},
  {"openCachedResultSet", openCachedResultSet, METH_VARARGS},
  {"openCachedResultSetWithOffset", openCachedResultSetWithOffset, METH_VARARGS},
  {"rowCount", rowCount, METH_VARARGS},
  {"colCount", colCount, METH_VARARGS},
  {"totalRows", totalRows, METH_VARARGS},
  {"affectedRows", affectedRows, METH_VARARGS},
  {"firstRowIndex", firstRowIndex, METH_VARARGS},
  {"getNullsAsEmptyStrings", getNullsAsEmptyStrings, METH_VARARGS},
  {"getNullsAsNone", getNullsAsNone, METH_VARARGS},
  {"getFieldByIndex", getFieldByIndex, METH_VARARGS},
  {"getFieldByName", getFieldByName, METH_VARARGS},
  {"getColumnTypeByIndex", getColumnTypeByIndex, METH_VARARGS},
  {"getColumnTypeByName", getColumnTypeByName, METH_VARARGS},
  {"getColumnName", getColumnName, METH_VARARGS},
  {"getColumnNames", getColumnNames, METH_VARARGS},
  {"free", sqlr_free, METH_VARARGS},
  {"endSession", endSession, METH_VARARGS},
  {"suspendSession", suspendSession, METH_VARARGS},
  {"resumeSession", resumeSession, METH_VARARGS},
  {"resumeCachedSession", resumeCachedSession, METH_VARARGS},
  {"getConnectionPort", getConnectionPort, METH_VARARGS},
  {"getConnectionSocket", getConnectionSocket, METH_VARARGS},
  {"getCacheFileOffset", getCacheFileOffset, METH_VARARGS},
  {"getColumnLengthByIndex", getColumnLengthByIndex, METH_VARARGS},
  {"getColumnLengthByName", getColumnLengthByName, METH_VARARGS},
  {"getLongestByIndex", getLongestByIndex, METH_VARARGS},
  {"getLongestByName", getLongestByName, METH_VARARGS},
  {"getRow", getRow, METH_VARARGS},
  {"getRowDictionary", getRowDictionary, METH_VARARGS},
  {"getRowRange", getRowRange, METH_VARARGS},
  {"errorMessage", errorMessage, METH_VARARGS},
  {"debugOn", debugOn, METH_VARARGS},
  {"debugOff", debugOff, METH_VARARGS},
  {"getDebug", getDebug, METH_VARARGS},
  {NULL,      NULL}        /* Sentinel */
};

void
initCSQLRelay()
{
  (void) Py_InitModule("CSQLRelay", SQLRMethods);
}

