// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <config.h>
#include <defaults.h>

#include <sqlrclient.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>

#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif
#include <time.h>

class	environment {
	public:
			environment();
		int	color;
		int	headers;
		int	stats;
		int	debug;
		int	final;
};

environment::environment() {
	color=0;
	headers=1;
	stats=1;
	debug=0;
	final=0;
}

class	sqlrsh {
	public:
		void	execute(int argc, char **argv);
	private:
		void	startupMessage(environment *env,
					char *host, int port, char *user);
		void	systemRcFile(sqlrclient *sqlrc, environment *env);
		void	userRcFile(sqlrclient *sqlrc, environment *env);
		void	runScript(sqlrclient *sqlrc, environment *env, 
						char *file, int returnerror);
		int	getCommandFromFile(ifstream *file, 
					strstream *cmdbuffer);
		int	runCommand(sqlrclient *sqlrc, environment *env, 
						char *command);
		int	commandType(char *command);
		void	internalCommand(environment *env, char *command);
		void	externalCommand(sqlrclient *sqlrc, environment *env, 
						char *command);
		void	initStats(environment *env);
		void	displayError(sqlrclient *sqlrc, environment *env);
		void	displayHeader(sqlrclient *sqlrc, environment *env);
		void	displayResultSet(sqlrclient *sqlrc, environment *env);
		void	displayStats(sqlrclient *sqlrc, environment *env);
		void	displayHelp(environment *env);
		void	interactWithUser(sqlrclient *sqlrc, environment *env);
		void	prompt(environment *env, int promptcount);
		void	error(char *errstring);
		void	addLineToCommand(strstream *command, strstream *line,
						int final);

		void	setColor(environment *env, int value);
		void	black(environment *env);
		void	red(environment *env);
		void	green(environment *env);
		void	yellow(environment *env);
		void	blue(environment *env);
		void	magenta(environment *env);
		void	cyan(environment *env);
		void	white(environment *env);
};

void	sqlrsh::systemRcFile(sqlrclient *sqlrc, environment *env) {

	// build file name
	strstream	*systemrcfile=new strstream();
	*systemrcfile << SYSTEM_SQLRSHRC << ends;

	// process the file
	runScript(sqlrc,env,systemrcfile->str(),0);

	// clean up
	delete systemrcfile;
}

void	sqlrsh::userRcFile(sqlrclient *sqlrc, environment *env) {

	// get user's home directory
	char	*home=home=getenv("HOME");
	if (!home) {
		home="~";
	}

	// build rcfilename
	strstream	*userrcfile=new strstream();
	*userrcfile << home << "/.sqlrshrc" << ends;

	// process the file
	runScript(sqlrc,env,userrcfile->str(),0);

	// clean up
	delete userrcfile;
}

void	sqlrsh::runScript(sqlrclient *sqlrc, environment *env, char *file,
							int returnerror) {

	// open the file
	ifstream	*scriptfile=new ifstream(file);
	if (scriptfile && scriptfile->good()) {

		while (1) {
		
			// get a command
			strstream	*command=new strstream();
			if (!getCommandFromFile(scriptfile,command)) {
				break;
			}

			// run the command
			runCommand(sqlrc,env,command->str());

			// clean up
			delete command;
		}

		// close the file
		scriptfile->close();
	} else {

		// error message
		if (returnerror) {
			strstream	*errmesg=new strstream();
			*errmesg << "Couldn't open file: " << file << ends;
			error(errmesg->str());
			delete errmesg;
		}
	}

	// clean up
	if (scriptfile) {
		delete scriptfile;
	}
}

int	sqlrsh::getCommandFromFile(ifstream *file, strstream *cmdbuffer) {

	char	character;
	
	while (1) {

		// get a character from the file
		character=file->rdbuf()->sgetc();

		// look for end of file
		if (character==-1) {
			*cmdbuffer << ends;
			return 0;
		}

		// look for an escape character
		if (character=='\\') {
			file->rdbuf()->snextc();
			character=file->rdbuf()->sgetc();
			*cmdbuffer << character;
			file->rdbuf()->snextc();
		}

		// look for an end of command delimiter
		if (character==';') {
			*cmdbuffer << ends;
			file->rdbuf()->snextc();
			return 1;
		}

		// write character to buffer and move on
		*cmdbuffer << character;
		file->rdbuf()->snextc();
	}
		
}

int	sqlrsh::runCommand(sqlrclient *sqlrc, environment *env, char *command) {

	int	cmdtype=commandType(command);

	if (cmdtype>0) {
		// if the command an internal command, run it as one
		internalCommand(env,command);
		return 1;
	} else if (cmdtype==0) {
		// if the command is not an internal command, 
		// execute it as a query and display the result set
		externalCommand(sqlrc,env,command);
		return 1;
	} else {
		return 0;
	}
}

int	sqlrsh::commandType(char *command) {

	// skip white space
	char	*ptr=command;
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// compare to known internal commands
	if (!strncasecmp(ptr,"color",5) ||
		!strncasecmp(ptr,"headers",7) ||
		!strncasecmp(ptr,"stats",5) ||
		!strncasecmp(ptr,"debug",5) ||
		!strncasecmp(ptr,"final",5) ||
		!strncasecmp(ptr,"help",4)) {

		// return value of 1 is internal command
		return 1;
	}

	// look for an exit command
	if (!strncasecmp(ptr,"quit",4) ||
		!strncasecmp(ptr,"exit",4)) {
		return -1;
	}

	// return value of 0 is external command
	return 0;
}

void	sqlrsh::internalCommand(environment *env, char *command) {

	// skip white space
	char	*ptr=command;
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// compare to known internal commands
	int	cmdtype=0;
	if (!strncasecmp(ptr,"color",5)) {
		ptr=ptr+5;
		cmdtype=1;
	} else if (!strncasecmp(ptr,"headers",7)) {
		ptr=ptr+7;
		cmdtype=2;
	} else if (!strncasecmp(ptr,"stats",5)) {	
		ptr=ptr+5;
		cmdtype=3;
	} else if (!strncasecmp(ptr,"debug",5)) {	
		ptr=ptr+5;
		cmdtype=4;
	} else if (!strncasecmp(ptr,"final",5)) {	
		ptr=ptr+5;
		cmdtype=5;
	} else if (!strncasecmp(ptr,"help",4)) {	
		displayHelp(env);
		return;
	} else {
		return;
	}

	// skip white space
	while (*ptr==' ' || *ptr=='	' || *ptr=='\n') {
		*ptr++;
	}

	// on or off?
	int	toggle;
	if (!strncasecmp(ptr,"on",2)) {
		toggle=1;
	} else if (!strncasecmp(ptr,"off",3)) {
		toggle=0;
	} else {
		return;
	}

	// set parameter
	if (cmdtype==1) {
		env->color=toggle;
	} else if (cmdtype==2) {
		env->headers=toggle;
	} else if (cmdtype==3) {
		env->stats=toggle;
	} else if (cmdtype==4) {
		env->debug=toggle;
	} else if (cmdtype==5) {
		env->final=toggle;
	}
}

void	sqlrsh::externalCommand(sqlrclient *sqlrc, environment *env, 
							char *command) {

	// init stats
	initStats(env);

	// handle debug
	if (env->debug) {
		sqlrc->debugOn();
	}

	// send the query
	sqlrc->sendQuery(command);
	if (!env->final) {
		sqlrc->endSession();
	}

	// look for an error
	if (sqlrc->errorMessage()) {

		// display the error
		displayError(sqlrc,env);

	} else {

		// display the header
		displayHeader(sqlrc,env);

		// display the result set
		displayResultSet(sqlrc,env);
	}

	// set debug back off
	sqlrc->debugOff();

	// display statistics
	displayStats(sqlrc,env);
}

void	sqlrsh::initStats(environment *env) {

	if (!env->stats) {
		return;
	}

	// call clock here or something
	clock();
}

void	sqlrsh::displayError(sqlrclient *sqlrc, environment *env) {

	cyan(env);
	cout << sqlrc->errorMessage() << endl << endl;
	white(env);
}

void	sqlrsh::displayHeader(sqlrclient *sqlrc, environment *env) {

	if (!env->headers) {
		return;
	}

	// display column names
	int	charcount=0;
	int	colcount=sqlrc->colCount();
	char	*name;
	int	namelen;
	int	longest;

	// iterate through columns
	for (int i=0; i<sqlrc->colCount(); i++) {

		// write the column name
		if (i%2==1) {
			green(env);
		} else {
			yellow(env);
		}
		name=sqlrc->getColumnName(i);
		cout << name;
		white(env);

		// which is longer, field name or longest field
		namelen=strlen(name);
		longest=sqlrc->getLongest(i);
		if (namelen>longest) {
			longest=namelen;
		}
		charcount=charcount+longest;

		// pad after the name with spaces
		for (int j=namelen; j<longest; j++) {
			cout << " ";
		}

		// put an extra space between names
		if (i<colcount-1) {
			cout << " ";
			charcount=charcount+1;
		}
	}
	cout << endl;

	// display delimiter
	red(env);
	for (int i=0; i<charcount; i++) {
		cout << "=";
	}
	cout << endl;
	white(env);
}

void	sqlrsh::displayResultSet(sqlrclient *sqlrc, environment *env) {

	// display column names
	int	colcount=sqlrc->colCount();
	int	namelen;
	int	longest;

	for (int i=0; i<sqlrc->rowCount(); i++) {
		for (int j=0; j<colcount; j++) {

			// write the column value
			if (i%2==1) {
				cyan(env);
			} else {
				white(env);
			}
			cout << sqlrc->getField(i,j);
			white(env);

			// which is longer, field name or longest field
			longest=sqlrc->getLongest(j);
			if (env->headers) {
				namelen=strlen(sqlrc->getColumnName(j));
				if (namelen>longest) {
					longest=namelen;
				}
			}

			// pad after the name with spaces
			for (int k=strlen(sqlrc->getField(i,j)); 
						k<longest; k++) {
				cout << " ";
			}

			// put an extra space between names
			if (j<colcount-1) {
				cout << " ";
			}
		}
		cout << endl;
	}
}

void	sqlrsh::displayStats(sqlrclient *sqlrc, environment *env) {

	if (!env->stats) {
		return;
	}

	// call clock again, display results
	red(env);
	cout << "	Rows Returned   : ";
	magenta(env);
	cout << sqlrc->rowCount() << endl;
	red(env);
	cout << "	Fields Returned : ";
	magenta(env);
	cout << sqlrc->rowCount()*sqlrc->colCount() << endl;
	red(env);
	cout << "	System time     : ";
	magenta(env);
	cout << clock() << endl;
	white(env);
}

void	sqlrsh::displayHelp(environment *env) {

	cout << endl;
	yellow(env);
	cout << "	To run a query, simply type it at the prompt," << endl;
	cout << "	followed by a semicolon.  Queries may be " << endl;
	cout << "	split over multiple lines." << endl;
	cout << endl;
	cyan(env);
	cout << "		color on/off	- ";
	green(env);
	cout << "toggles colorizing" << endl;
	cyan(env);
	cout << "		headers on/off	- ";
	green(env);
	cout << "toggles column descriptions before result set" << endl;
	cyan(env);
	cout << "		stats on/off	- ";
	green(env);
	cout << "toggles statistics after result set" << endl;
	cyan(env);
	cout << "		debug on/off	- ";
	green(env);
	cout << "toggles debug messages" << endl;
	cyan(env);
	cout << "		final on/off	- ";
	green(env);
	cout << "toggles use of one session per query" << endl;
	cyan(env);
	cout << "		exit/quit	- ";
	green(env);
	cout << "exits" << endl;
	cout << endl;
	yellow(env);
	cout << "	All commands must be followed by a semicolon.";
	cout << endl;
	white(env);
}

void	sqlrsh::startupMessage(environment *env,
				char *host, int port, char *user) {

	red(env);
	cout << "SQLRShell - ";
	green(env);
	cout << "Version " << "0.22" << endl;
	yellow(env);
	cout << "	Connected to: ";
	blue(env);
	cout << host << ":" << port << " as ";
	cout << user << endl << endl;
	yellow(env);
	cout << "	type help; for a help." << endl << endl;
	white(env);
}

void	sqlrsh::interactWithUser(sqlrclient *sqlrc, environment *env) {

	// init some variables
	char		buffer1;
	char		buffer2;
	strstream	*command;
	int		exit=0;
	int		promptcount;

	while (!exit) {

		// prompt the user
		promptcount=0;
		prompt(env,promptcount);
		
		// get the command
		command=new strstream();
		while (1) {
			cin.read(&buffer1,1);
			if (buffer1==';') {
				cin.read(&buffer2,1);
				if (buffer2=='\n') {
					*command << ends;
					break;
				} else {
					*command << buffer1;
				}
			} else if (buffer1=='\n') {
				promptcount++;
				*command << " ";
				prompt(env,promptcount);
			} else {
				*command << buffer1;
			}
		}

		// run the command
		if (!runCommand(sqlrc,env,command->str())) {	
			exit=1;
		}

		delete command;
	}
}

void	sqlrsh::prompt(environment *env, int promptcount) {

	cout << promptcount << "> ";
}

void	sqlrsh::error(char *errstring) {

	// print the error
	cerr << errstring << endl;
}

void	sqlrsh::execute(int argc, char **argv) {

	// usage message
	if (argc<5) {
		cout << "usage:  sqlrsh  host port socket user password [script]";
		cout << endl;
		exit(0);
	}

	// set connection variables
	char	*host=argv[1];
	int	port=atoi(argv[2]);
	char	*socket=argv[3];
	char	*user=argv[4];
	char	*password=argv[5];

	// connect to sql relay
	sqlrclient	*sqlrc=new sqlrclient(host,port,socket,user,password,
									0,1);

	// set up an environment
	environment	*env=new environment();

	// process RC files
	systemRcFile(sqlrc,env);
	userRcFile(sqlrc,env);

	// if a script was specified, run it otherwise go into interactive mode
	if (argv[6]) {
		runScript(sqlrc,env,argv[6],1);
	} else {
		startupMessage(env,host,port,user);
		interactWithUser(sqlrc,env);
	}

	// clean up
	delete sqlrc;
	delete env;
}

void	sqlrsh::setColor(environment *env, int value) {
	if (env->color) {
		cout << "\033[0;" << value << "m";
	}
}

void	sqlrsh::black(environment *env) {
	setColor(env,30);
}

void	sqlrsh::red(environment *env) {
	setColor(env,31);
}

void	sqlrsh::green(environment *env) {
	setColor(env,32);
}

void	sqlrsh::yellow(environment *env) {
	setColor(env,33);
}

void	sqlrsh::blue(environment *env) {
	setColor(env,34);
}

void	sqlrsh::magenta(environment *env) {
	setColor(env,35);
}

void	sqlrsh::cyan(environment *env) {
	setColor(env,36);
}

void	sqlrsh::white(environment *env) {
	setColor(env,37);
}



int	main(int argc, char **argv) {

	#include <version.h>

	sqlrsh	s;
	s.execute(argc,argv);
}
