// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <db2connection.h>

#include <datatypes.h>

#include <stdlib.h>

db2connection::db2connection() {
	errormsg=NULL;
}

db2connection::~db2connection() {
	if (errormsg) {
		delete errormsg;
	}
}

int	db2connection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	db2connection::handleConnectString() {
	server=connectStringValue("server");
	user=connectStringValue("user");
	password=connectStringValue("password");
}

int	db2connection::logIn() {

	// allocate environment handle
	erg=SQLAllocHandle(SQL_HANDLE_ENV,SQL_NULL_HANDLE,&env);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	// allocate connection handle
	erg=SQLAllocHandle(SQL_HANDLE_DBC,env,&dbc);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	// set the connect timeout
	SQLSetConnectAttr(dbc,SQL_LOGIN_TIMEOUT,(SQLPOINTER *)5,0);

	// connect to the database
	erg=SQLConnect(dbc,(SQLCHAR *)server,SQL_NTS,
				(SQLCHAR *)user,SQL_NTS,
				(SQLCHAR *)password,SQL_NTS);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	return 1;
}

void	db2connection::logOut() {
	SQLDisconnect(dbc);
	SQLFreeHandle(SQL_HANDLE_DBC,dbc);
	SQLFreeHandle(SQL_HANDLE_ENV,env);
}

int	db2connection::prepareQuery(char *query, long length) {

	// allocate the statement
	erg=SQLAllocHandle(SQL_HANDLE_STMT,dbc,&stmt);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	// prepare the query
	erg=SQLPrepare(stmt,(SQLCHAR *)query,length);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	return 1;
}

int	db2connection::inputBindValue(char *variable,
					short variablesize,
					char *value,
					short valuesize,
					short *isnull) {

	if (*isnull==SQL_NULL_DATA) {
		erg=SQLBindParameter(stmt,
				atoi(variable+1),
				SQL_PARAM_INPUT,
				SQL_C_CHAR,
				SQL_CHAR,
				0,
				0,
				value,
				valuesize,
				(SQLINTEGER *)isnull);
	} else {
		erg=SQLBindParameter(stmt,
				atoi(variable+1),
				SQL_PARAM_INPUT,
				SQL_C_CHAR,
				SQL_CHAR,
				0,
				0,
				value,
				valuesize,
				(SQLINTEGER *)NULL);
	}
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}
	return 1;
}

int	db2connection::inputBindValue(char *variable,
					short variablesize,
					long *value) {

	erg=SQLBindParameter(stmt,
				atoi(variable+1),
				SQL_PARAM_INPUT,
				SQL_C_LONG,
				SQL_INTEGER,
				0,
				0,
				value,
				sizeof(long),
				(SQLINTEGER *)NULL);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}
	return 1;
}

int	db2connection::inputBindValue(char *variable,
					short variablesize,
					double *value,
					short precision,
					short scale) {

	erg=SQLBindParameter(stmt,
				atoi(variable+1),
				SQL_PARAM_INPUT,
				SQL_C_DOUBLE,
				SQL_DECIMAL,
				precision,
				scale,
				value,
				sizeof(double),
				(SQLINTEGER *)NULL);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}
	return 1;
}

int	db2connection::outputBindValue(char *variable, short variablesize,
				char *value, short valuesize, short *isnull) {

	erg=SQLBindParameter(stmt,
				atoi(variable+1),
				SQL_PARAM_OUTPUT,
				SQL_C_CHAR,
				SQL_CHAR,
				0,
				0,
				value,
				valuesize,
				(SQLINTEGER *)isnull);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}
	return 1;
}

short	db2connection::nonNullBindValue() {
	return 0;
}

short	db2connection::nullBindValue() {
	return SQL_NULL_DATA;
}

int	db2connection::executeQuery(char *query, long length) {

	// initialize counts
	ncols=0;

	// execute the query
	erg=SQLExecute(stmt);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	// get the column count
	erg=SQLNumResultCols(stmt,&ncols);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	// allocate column name buffers
	col=new column[ncols];

	// run through the columns
	for (int i=0; i<ncols; i++) {

		// column name
		erg=SQLColAttribute(stmt,i+1,SQL_COLUMN_LABEL,
				col[i].name,MAX_ITEM_BUFFER_SIZE,
				(SQLSMALLINT *)&(col[i].namelength),NULL);
		if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
			return 0;
		}

		// column length
		erg=SQLColAttribute(stmt,i+1,SQL_COLUMN_LENGTH,
				NULL,0,NULL,&(col[i].length));
		if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
			return 0;
		}

		// column type
		erg=SQLColAttribute(stmt,i+1,SQL_COLUMN_TYPE,
				NULL,0,NULL,&(col[i].type));
		if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
			return 0;
		}
	}

	// get the row count
	erg=SQLRowCount(stmt,&affectedrows);
	if (erg!=SQL_SUCCESS && erg!=SQL_SUCCESS_WITH_INFO) {
		return 0;
	}

	return 1;
}

void	db2connection::commit() {
	SQLEndTran(SQL_HANDLE_ENV,env,SQL_COMMIT);
}

void	db2connection::rollback() {
	SQLEndTran(SQL_HANDLE_ENV,env,SQL_ROLLBACK);
}

char	*db2connection::getErrorMessage(int *liveconnection) {

	SQLCHAR		error[501];
	SQLCHAR		state[10];
	SQLINTEGER	nativeerrnum;
	SQLSMALLINT	errnum;

	// need to use SQLGetDiagRec and SQLGetDiagField here...
	SQLError(env,dbc,stmt,state,&nativeerrnum,error,500,&errnum);
	if (errormsg) {
		delete errormsg;
	}
	errormsg=new strstream();
	*errormsg << error << ends;

	return errormsg->str();
}

void	db2connection::returnHeader() {

	// send row counts
	sendRowCounts((long)-1,(long)affectedrows);

	// a useful variable
	int	type;

	// for each column...
	for (int i=0; i<ncols; i++) {

		if (col[i].type==SQL_BIGINT) {
			type=BIGINT_DATATYPE;
		} else if (col[i].type==SQL_BINARY) {
			type=BINARY_DATATYPE;
		} else if (col[i].type==SQL_BIT) {
			type=BIT_DATATYPE;
		} else if (col[i].type==SQL_CHAR) {
			type=CHAR_DATATYPE;
		} else if (col[i].type==SQL_DATE) {
			type=DATE_DATATYPE;
		} else if (col[i].type==SQL_DECIMAL) {
			type=DECIMAL_DATATYPE;
		} else if (col[i].type==SQL_DOUBLE) {
			type=DOUBLE_DATATYPE;
		} else if (col[i].type==SQL_FLOAT) {
			type=FLOAT_DATATYPE;
		} else if (col[i].type==SQL_INTEGER) {
			type=INTEGER_DATATYPE;
		} else if (col[i].type==SQL_LONGVARBINARY) {
			type=LONGVARBINARY_DATATYPE;
		} else if (col[i].type==SQL_LONGVARCHAR) {
			type=LONGVARCHAR_DATATYPE;
		} else if (col[i].type==SQL_NUMERIC) {
			type=NUMERIC_DATATYPE;
		} else if (col[i].type==SQL_REAL) {
			type=REAL_DATATYPE;
		} else if (col[i].type==SQL_SMALLINT) {
			type=SMALLINT_DATATYPE;
		} else if (col[i].type==SQL_TIME) {
			type=TIME_DATATYPE;
		} else if (col[i].type==SQL_TIMESTAMP) {
			type=TIMESTAMP_DATATYPE;
		} else if (col[i].type==SQL_TINYINT) {
			type=TINYINT_DATATYPE;
		} else if (col[i].type==SQL_VARBINARY) {
			type=VARBINARY_DATATYPE;
		} else if (col[i].type==SQL_VARCHAR) {
			type=VARCHAR_DATATYPE;
		// DB2 has more datatypes than ODBC...
		} else if (col[i].type==SQL_GRAPHIC) {
			type=GRAPHIC_DATATYPE;
		} else if (col[i].type==SQL_VARGRAPHIC) {
			type=VARGRAPHIC_DATATYPE;
		} else if (col[i].type==SQL_LONGVARGRAPHIC) {
			type=LONGVARGRAPHIC_DATATYPE;
		} else if (col[i].type==SQL_BLOB) {
			type=BLOB_DATATYPE;
		} else if (col[i].type==SQL_CLOB) {
			type=CLOB_DATATYPE;
		} else if (col[i].type==SQL_DBCLOB) {
			type=DBCLOB_DATATYPE;
		} else if (col[i].type==SQL_DATALINK) {
			type=DATALINK_DATATYPE;
		} else if (col[i].type==SQL_USER_DEFINED_TYPE) {
			type=USER_DEFINED_TYPE_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send column definition
		sendColumnDefinition(col[i].name,col[i].namelength,
					type,col[i].length);
	}
}

int	db2connection::noRowsToReturn() {

	// if there are no columns, then there can't be any rows either
	if (ncols==0) {
		return 1;
	}
	return 0;
}

int	db2connection::fetchRow() {

	erg=SQLFetch(stmt);
	if (erg==SQL_NO_DATA) {
		return 0;
	}
	return 1;
}

void	db2connection::returnRow() {

	for (int index=0; index<ncols; index++) {

		// initialize the buffer to a NULL string
		field[0]=NULL;

		// get the field
		SQLGetData(stmt,index+1,SQL_C_CHAR,
			(SQLPOINTER)field,MAX_ITEM_BUFFER_SIZE,&indicator);

		// handle a null field
		if (indicator==SQL_NULL_DATA) {
			sendNullField();
			continue;
		}

		// handle a non-null field
		sendField(field,strlen(field));
	}
}

void	db2connection::cleanUpData() {
	SQLFreeHandle(SQL_HANDLE_STMT,stmt);
	delete[] col;
}

int	db2connection::ping() {
	return 1;
}

char	*db2connection::identify() {
	return "db2";
}
