// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <freetdsconnection.h>

#include <datatypes.h>

#include <stdlib.h>

#ifdef __GNUC__
static	strstream	*freetdsconnection::errorstring;
static	int		freetdsconnection::deadconnection;
#endif


freetdsconnection::freetdsconnection() {
	context=(CS_CONTEXT *)NULL;
	errorstring=NULL;

	// define conversion format, will need it alot later
	convfmt=new CS_DATAFMT;
	convfmt->datatype=CS_CHAR_TYPE;
	convfmt->locale=NULL;
	convfmt->format=CS_FMT_NULLTERM;
	convfmt->maxlength=0;
	convfmt->scale=0;
	convfmt->precision=0;
}

freetdsconnection::~freetdsconnection() {
	if (errorstring) {
		delete errorstring;
	}

	delete convfmt;
}

int	freetdsconnection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	freetdsconnection::handleConnectString() {
	user=connectStringValue("user");
	password=connectStringValue("password");
	server=connectStringValue("server");
	charset=connectStringValue("charset");
	language=connectStringValue("language");
	encryption=connectStringValue("encryption");
	hostname=connectStringValue("hostname");
	packetsize=connectStringValue("packetsize");
}

int	freetdsconnection::logIn() {

	// allocate a context
	if (cs_ctx_alloc(CS_VERSION_100,&context)!=CS_SUCCEED) {
		return 0;
	}
	// init the context
	if (ct_init(context,CS_VERSION_100)!=CS_SUCCEED) {
		return 0;
	}


	// configure the error handling callbacks
	if (ct_callback(context,conn,CS_SET,CS_CLIENTMSG_CB,
		(CS_VOID *)freetdsconnection::clientMessageCallback)
			!=CS_SUCCEED) {
		return 0;
	}
	if (ct_callback(context,conn,CS_SET,CS_SERVERMSG_CB,
		(CS_VOID *)freetdsconnection::serverMessageCallback)
			!=CS_SUCCEED) {
		return 0;
	}


	// allocate a connection
	if (ct_con_alloc(context,&conn)!=CS_SUCCEED) {
		return 0;
	}


	// set the user/password to use
	if (ct_con_props(conn,CS_SET,CS_USERNAME,user,
			CS_NULLTERM,(CS_INT *)NULL)!=CS_SUCCEED) {
		return 0;
	}
	if (ct_con_props(conn,CS_SET,CS_PASSWORD,password,
			CS_NULLTERM,(CS_INT *)NULL)!=CS_SUCCEED) {
		return 0;
	}

	// set application name
	if (ct_con_props(conn,CS_SET,CS_APPNAME,(CS_VOID *)"sqlrelay",
			CS_NULLTERM,(CS_INT *)NULL)!=CS_SUCCEED) {
		return 0;
	}

	// set hostname
	if (hostname && hostname[0]) {
		if (ct_con_props(conn,CS_SET,CS_HOSTNAME,(CS_VOID *)hostname,
				CS_NULLTERM,(CS_INT *)NULL)!=CS_SUCCEED) {
			return 0;
		}
	}

	// set packetsize
	if (packetsize && packetsize[0]) {
		if (ct_con_props(conn,CS_SET,CS_PACKETSIZE,
				(CS_VOID *)atoi(packetsize),
				CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
			return 0;
		}
	}

	// set encryption
	if (encryption && atoi(encryption)==1) {
		enc=CS_TRUE;
		if (ct_con_props(conn,CS_SET,CS_PACKETSIZE,
			(CS_VOID *)&enc,
			CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
			return 0;
		}
	}

	// init locale
	locale=NULL;
	if (cs_loc_alloc(context,&locale)!=CS_SUCCEED) {
		return 0;
	}
	if (cs_locale(context,CS_SET,locale,CS_LC_ALL,(CS_CHAR *)NULL,
			CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
		return 0;
	}

	// set language
	if (language && language[0]) {
		if (cs_locale(context,CS_SET,locale,CS_SYB_LANG,
			(CS_CHAR *)language,CS_NULLTERM,(CS_INT *)NULL)!=
				CS_SUCCEED) {
			return 0;
		}
	}

	// set charset
	if (charset && charset[0]) {
		if (cs_locale(context,CS_SET,locale,CS_SYB_CHARSET,
			(CS_CHAR *)charset,CS_NULLTERM,(CS_INT *)NULL)!=
				CS_SUCCEED) {
			return 0;
		}
	}

	// set locale
	if (ct_con_props(conn,CS_SET,CS_LOC_PROP,(CS_VOID *)locale,
			CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
		return 0;
	}
	

	// connect to the database
	if (ct_connect(conn,server,CS_NULLTERM)!=CS_SUCCEED) {
		return 0;
	}

	// allocate a command structure
	if (ct_cmd_alloc(conn,&cmd)!=CS_SUCCEED) {
		return 0;
	}

	return 1;
}

void	freetdsconnection::logOut() {
	ct_cmd_drop(cmd);
	cs_loc_drop(context,locale);
	ct_close(conn,CS_UNUSED);
	ct_con_drop(conn);
	ct_exit(context,CS_UNUSED);
	cs_ctx_drop(context);
}

int	freetdsconnection::executeQuery(char *query, long length) {

	// initiate a language command
	if (ct_command(cmd,CS_LANG_CMD,query,length,
			CS_UNUSED)!=CS_SUCCEED) {
		return 0;
	}

	// clear out any errors
	if (errorstring) {
		deadconnection=0;
		delete errorstring;
		errorstring=NULL;
	}

	// initialize return values
	ncols=0;
	affectedrows=0;

	// send the command
	if (ct_send(cmd)!=CS_SUCCEED) {
		return 0;
	}

	// Get the results. Sybase is weird... A query can return multiple
	// result sets?  We're only interested in the first one,
	// the rest will be cancelled.
	if (ct_results(cmd,&results_type)==CS_FAIL) {
		// return a dead database on failre
		ct_cancel(conn,cmd,CS_CANCEL_ALL);
		deadconnection=1;
		return 0;
	}

	// handle a failed command
	if ((int)results_type==CS_CMD_FAIL) {
		ct_cancel(conn,cmd,CS_CANCEL_ALL);
		return 0;
	}

	// get the column count, but only for queries which return rows
	if (results_type==CS_ROW_RESULT) {
		if (ct_res_info(cmd,CS_NUMDATA,(CS_VOID *)&ncols,
				CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
			return 0;
		}
	}

	// get the affected row count, but only for queries which 
	// return "succeed"
	/*if (results_type==CS_CMD_SUCCEED) {
		if (ct_res_info(cmd,CS_ROW_COUNT,(CS_VOID *)&affectedrows,
				CS_UNUSED,(CS_INT *)NULL)!=CS_SUCCEED) {
			return 0;
		} 
	} else {
		affectedrows=0;
	}*/

	// I'd like to make that code up there work, but it always crashes
	// (or just returns 0 in version 0.51) so it's dummied up for now.
	affectedrows=-1;

	// return success only if no error was generated
	if (errorstring) {
		return 0;
	} else {
		return 1;
	}
}

char	*freetdsconnection::getErrorMessage(int *liveconnection) {
	if (deadconnection) {
		*liveconnection=0;
	} else {
		*liveconnection=1;
	}
	if (errorstring) {
		return errorstring->str();
	} else {
		return NULL;
	}
}

void	freetdsconnection::returnHeader() {

	initcol=0;

	// send row counts (actual row count unknown in freetds)
	sendRowCounts((long)-1,(long)affectedrows);

	// unless the query was a successful select, send no header
	if (results_type!=CS_ROW_RESULT) {
		return;
	}

	// gonna need these later
	int	length;
	int	type;

	initColumns();

	// for each column...
	for (int i=0; i<(int)ncols; i++) {

		// get the column description
		if (ct_describe(cmd,i+1,column[i])!=CS_SUCCEED) {
			break;
		}
	
		// set the datatype
		if (column[i]->datatype==CS_CHAR_TYPE) {
			type=CHAR_DATATYPE;
		} else if (column[i]->datatype==CS_INT_TYPE) {
			column[i]->maxlength = 12;
			type=INT_DATATYPE;
		} else if (column[i]->datatype==CS_SMALLINT_TYPE) {
			column[i]->maxlength = 7;
			type=SMALLINT_DATATYPE;
		} else if (column[i]->datatype==CS_TINYINT_TYPE) {
			column[i]->maxlength = 4;
			type=TINYINT_DATATYPE;
		} else if (column[i]->datatype==CS_MONEY_TYPE) {
			column[i]->maxlength = 24;
			type=MONEY_DATATYPE;
		} else if (column[i]->datatype==CS_DATETIME_TYPE) {
			column[i]->maxlength = 30;
			type=DATETIME_DATATYPE;
		} else if (column[i]->datatype==CS_NUMERIC_TYPE) {
			column[i]->maxlength = column[i]->precision + 2;
			type=NUMERIC_DATATYPE;
		} else if (column[i]->datatype==CS_DECIMAL_TYPE) {
			column[i]->maxlength = column[i]->precision + 2;
			type=DECIMAL_DATATYPE;
		} else if (column[i]->datatype==CS_DATETIME4_TYPE) {
			column[i]->maxlength = 30;
			type=DATETIME4_DATATYPE;
		} else if (column[i]->datatype==CS_MONEY4_TYPE) {
			column[i]->maxlength = 24;
			type=MONEY4_DATATYPE;
		} else if (column[i]->datatype==CS_IMAGE_TYPE) {
			type=IMAGE_DATATYPE;
		} else if (column[i]->datatype==CS_BINARY_TYPE) {
			column[i]->maxlength *= 2;
			type=BINARY_DATATYPE;
		} else if (column[i]->datatype==CS_BIT_TYPE) {
			column[i]->maxlength = 4;
			type=BIT_DATATYPE;
		} else if (column[i]->datatype==CS_REAL_TYPE) {
			column[i]->maxlength = 24;
			type=REAL_DATATYPE;
		} else if (column[i]->datatype==CS_FLOAT_TYPE) {
			column[i]->maxlength = 24;
			type=FLOAT_DATATYPE;
		} else if (column[i]->datatype==CS_TEXT_TYPE) {
			type=TEXT_DATATYPE;
		} else if (column[i]->datatype==CS_VARCHAR_TYPE) {
			type=VARCHAR_DATATYPE;
		} else if (column[i]->datatype==CS_VARBINARY_TYPE) {
			column[i]->maxlength *= 2;
			type=VARBINARY_DATATYPE;
		} else if (column[i]->datatype==CS_LONGCHAR_TYPE) {
			type=LONGCHAR_DATATYPE;
		} else if (column[i]->datatype==CS_LONGBINARY_TYPE) {
			type=LONGBINARY_DATATYPE;
		} else if (column[i]->datatype==CS_LONG_TYPE) {
			type=LONG_DATATYPE;
		} else if (column[i]->datatype==CS_ILLEGAL_TYPE) {
			type=ILLEGAL_DATATYPE;
		} else if (column[i]->datatype==CS_SENSITIVITY_TYPE) {
			type=SENSITIVITY_DATATYPE;
		} else if (column[i]->datatype==CS_BOUNDARY_TYPE) {
			type=BOUNDARY_DATATYPE;
		} else if (column[i]->datatype==CS_VOID_TYPE) {
			type=VOID_DATATYPE;
		} else if (column[i]->datatype==CS_USHORT_TYPE) {
			type=USHORT_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send the column definition
		sendColumnDefinition(column[i]->name,
					strlen(column[i]->name),
					type,
					column[i]->maxlength);

		// allocate the data buffer
		data[i]=(CS_VOID *)malloc(column[i]->maxlength+1);

		// bind the column for the fetches
		if (ct_bind(cmd,i+1,column[i],(CS_VOID *)data[i],
				&datalength[i],&nullindicator[i])!=CS_SUCCEED) {
			break;
		}
	}
}

int	freetdsconnection::noRowsToReturn() {

	// unless the query was a successful select, send no data
	if (results_type!=CS_ROW_RESULT) {
		return 1;
	}
	return 0;
}

int	freetdsconnection::fetchRow() {
	return ct_fetch(cmd,CS_UNUSED,CS_UNUSED,CS_UNUSED,NULL)==CS_SUCCEED;
}

void	freetdsconnection::returnRow() {

	// some useful variables
	CS_INT 		len;
	char		*datastr;

	// send each row back
	for (int col=0; col<(int)ncols; col++) {

		if (!nullindicator[col]) {

			// transform data to a character string
			convfmt->maxlength=datalength[col];
			datastr=new char[datalength[col]+1];
			cs_convert(context,column[col],
					data[col],
					convfmt,
					(CS_VOID *)datastr,
					&len);
			if (datastr) {
				sendField(datastr,datalength[col]);
				delete[] datastr;
			} else {
				sendNullField();
			}
		} else {
			sendNullField();
		}
	}
}


void	freetdsconnection::cleanUpData() {
	if (initcol) {
		cleanUpColumns();
	}
	cancelRemainingResultSets();
}

void	freetdsconnection::initColumns() {

	initcol=1;

	// init a structure to hold the column info
	column=new CS_DATAFMT *[ncols];
	data=new CS_VOID *[ncols];
	datalength=new CS_INT[ncols];
	nullindicator=new CS_SMALLINT[ncols];

	for (int i=0; i<ncols; i++) {

		column[i]=new CS_DATAFMT;

		// init the column
		column[i]->name[0]='\0';
		column[i]->namelen=0;
		column[i]->datatype=0;
		column[i]->format=0;
		column[i]->maxlength=0;
		column[i]->scale=0;
		column[i]->precision=0;
		column[i]->status=0;
		column[i]->count=0;
		column[i]->usertype=0;
		column[i]->locale=NULL;

		data[i]=NULL;
		datalength[i]=0;
		nullindicator[i]=0;
	}
}

void	freetdsconnection::cleanUpColumns() {

	// clean up the column stuff
	for (int i=0; i<ncols; i++) {
		delete column[i];
		delete data[i];
	}
	delete[] column;
	delete[] data;
	delete[] datalength;
	delete[] nullindicator;
}

int	freetdsconnection::cancelRemainingResultSets() {

	// cancel the rest of the result sets
	CS_INT	return_code;
	while ((return_code=ct_results(cmd,&results_type))==CS_SUCCEED) {
		ct_cancel(conn,cmd,CS_CANCEL_CURRENT);
	}

	// return a dead database on absolute failre
	if (return_code==CS_FAIL) {
		ct_cancel(conn,cmd,CS_CANCEL_ALL);
		deadconnection=1;
		return 0;
	} else {
		return 1;
	}
}

CS_RETCODE	freetdsconnection::clientMessageCallback(CS_CONTEXT *ctxt, 
						CS_CONNECTION *cnn,
						CS_CLIENTMSG *emsgp) {
	if (errorstring) {
		delete errorstring;
	}
	errorstring=new strstream();

	*errorstring << "Client Library error:" << endl;
	*errorstring << "	severity(";
	*errorstring << (long)CS_SEVERITY(emsgp->severity) << ")" << endl;
	*errorstring << "	number(";
	*errorstring << (long)CS_SEVERITY(emsgp->msgnumber) << ")" << endl;
	*errorstring << "Error:" << endl;
	*errorstring << "	" << emsgp->msgstring;

	if (emsgp->osstringlen>0) {
		*errorstring << "Operating System Error:" << endl;
		*errorstring << endl << "	" << emsgp->osstring;
	}
	*errorstring << ends;

	// for a timeout message, set deadconnection to 1
	if (CS_SEVERITY(emsgp->msgnumber)==CS_SV_RETRY_FAIL &&
		CS_NUMBER(emsgp->msgnumber)==63 &&
		CS_ORIGIN(emsgp->msgnumber)==63 &&
		CS_LAYER(emsgp->msgnumber)==63) {
		deadconnection=1;
	}

	return CS_SUCCEED;
}

CS_RETCODE	freetdsconnection::serverMessageCallback(CS_CONTEXT *ctxt, 
						CS_CONNECTION *cnn,
						CS_SERVERMSG *msgp) {
	if (errorstring) {
		delete errorstring;
	}
	errorstring=new strstream();

	*errorstring << "Server message:" << endl;
	*errorstring << "	severity(";
	*errorstring << (long)CS_SEVERITY(msgp->severity) << ")" << endl;
	*errorstring << "	number(";
	*errorstring << (long)CS_SEVERITY(msgp->msgnumber) << ")" << endl;
	*errorstring << "	state(";
	*errorstring << (long)CS_SEVERITY(msgp->state) << ")" << endl;
	*errorstring << "	line(";
	*errorstring << (long)CS_SEVERITY(msgp->line) << ")" << endl;
	*errorstring << "Server Name:" << endl;
	*errorstring << "	" << msgp->svrname;
	*errorstring << "Procedure Name:" << endl;
	*errorstring << "	" << msgp->proc;
	*errorstring << "Error:" << endl;
	*errorstring << "	" << msgp->text;
	*errorstring << ends;

	return CS_SUCCEED;
}

int	freetdsconnection::ping() {

	if (!executeQuery("select 1",8)) {
		int	liveconnection;
		getErrorMessage(&liveconnection);
		if (liveconnection) {
			return 1;
		}
		return 0;
	}
	return 1;
}

char	*freetdsconnection::identify() {
	return "freetds";
}
