// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <interbaseconnection.h>

#include <datatypes.h>

#include <iomanip.h>

#include <stdlib.h>
#include <math.h>
#include <time.h>

interbaseconnection::interbaseconnection() {

	db=0L;
	tr=0L;

	outsqlda=(XSQLDA ISC_FAR *)malloc(XSQLDA_LENGTH(MAX_SELECT_LIST_SIZE));
	outsqlda->version=SQLDA_VERSION1;
	outsqlda->sqln=MAX_SELECT_LIST_SIZE;

	insqlda=(XSQLDA ISC_FAR *)malloc(XSQLDA_LENGTH(MAX_BIND_VARS));
	insqlda->version=SQLDA_VERSION1;
	insqlda->sqln=MAX_BIND_VARS;

	errormsg=NULL;

	querytype=0;
}

interbaseconnection::~interbaseconnection() {
	free(outsqlda);
	free(insqlda);
	if (errormsg) {
		delete errormsg;
	}
}

int	interbaseconnection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	interbaseconnection::handleConnectString() {
	database=connectStringValue("database");
	char	*dialectstr=connectStringValue("dialect");
	dialect=atoi(dialectstr);
	if (dialect<1) {
		dialect=1;
	}
	if (dialect>3) {
		dialect=3;
	}
	user=connectStringValue("user");
	password=connectStringValue("password");
}

int	interbaseconnection::logIn() {

	// initialize a dpb
	char	*dpbptr=dpb;
	*dpbptr++=isc_dpb_version1;
	dpblength=dpbptr-dpb;

	// handle user/password parameters
	if (user && user[0] && password && password[0]) {
		isc_expand_dpb(&dpbptr,&dpblength,
				isc_dpb_user_name,user,
				isc_dpb_password,password,
				NULL);
	}

	// attach to the database
	if (isc_attach_database(error,strlen(database),database,&db,
							dpblength,dpb)) {
		return 0;
	}

	// start a transaction
	if (isc_start_transaction(error,&tr,1,&db,0,NULL)) {
		return 0;
	}
	return 1;
}

void	interbaseconnection::logOut() {
	isc_detach_database(error,&db);
}

int	interbaseconnection::prepareQuery(char *query, long length) {

	// allocate a statement handle
	stmt=NULL;
	if (isc_dsql_allocate_statement(error,&db,&stmt)) {
		return 0;
	}

	// prepare the statement
	if (isc_dsql_prepare(error,&tr,&stmt,length,query,dialect,outsqlda)) {
		return 0;
	}

	// get the statement type
	char	typeitem[]={isc_info_sql_stmt_type};
	char	*resbuffer=new char[1024];
	if (isc_dsql_sql_info(error,&stmt,
				sizeof(typeitem),typeitem,
				1024,resbuffer)) {
		return 0;
	}

	int	len=isc_vax_integer(resbuffer+1,2);
	querytype=isc_vax_integer(resbuffer+3,len);

	// find bind parameters, if any
	insqlda->sqld=0;
	if (isc_dsql_describe_bind(error,&stmt,1,insqlda)) {
		return 0;
	}
	insqlda->sqln=insqlda->sqld;

	return 1;
}

int	interbaseconnection::inputBindValue(char *variable,
					short variablesize,
					char *value,
					short valuesize,
					short *isnull) {
	int	index=atoi(variable);
	insqlda->sqlvar[index].sqltype=SQL_TEXT+1;
	insqlda->sqlvar[index].sqldata=value;
	insqlda->sqlvar[index].sqlind=isnull;
	return 1;
}

int	interbaseconnection::inputBindValue(char *variable,
					short variablesize,
					long *value) {
	int	index=atoi(variable);
	insqlda->sqlvar[index].sqltype=SQL_LONG+1;
	(long *)insqlda->sqlvar[index].sqldata=value;
	insqlda->sqlvar[index].sqlind=0;
	return 1;
}

int	interbaseconnection::inputBindValue(char *variable,
					short variablesize,
					double *value,
					short precision,
					short scale) {
	int	index=atoi(variable);
	insqlda->sqlvar[index].sqltype=SQL_DOUBLE+1;
	(double *)insqlda->sqlvar[index].sqldata=value;
	insqlda->sqlvar[index].sqlind=0;
	return 1;
}

int	interbaseconnection::outputBindValue(char *variable, short variablesize,
				char *value, short valuesize, short *isnull) {
	int	index=atoi(variable);
	insqlda->sqlvar[index].sqltype=SQL_TEXT+1;
	insqlda->sqlvar[index].sqldata=(char *)value;
	insqlda->sqlvar[index].sqlind=isnull;
	return 1;
}

short	interbaseconnection::nonNullBindValue() {
	return 0;
}

short	interbaseconnection::nullBindValue() {
	return -1;
}

int	interbaseconnection::executeQuery(char *query, long length) {

	// describe the statement
	outsqlda->sqld=0;
	if (isc_dsql_describe(error,&stmt,1,outsqlda)) {
		return 0;
	}
	if (outsqlda->sqld>MAX_SELECT_LIST_SIZE) {
		outsqlda->sqld=MAX_SELECT_LIST_SIZE;
	}

	for (int i=0; i<outsqlda->sqld; i++) {

		// save the actual field type
		field[i].type=outsqlda->sqlvar[i].sqltype;

		// handle the null indicator
		outsqlda->sqlvar[i].sqlind=&field[i].nullindicator;

		// coerce the datatypes and point where the data should go
		if (outsqlda->sqlvar[i].sqltype==SQL_TEXT || 
				outsqlda->sqlvar[i].sqltype==SQL_TEXT+1) {
			outsqlda->sqlvar[i].sqldata=field[i].textbuffer;
			field[i].sqlrtype=TEXT_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_VARYING ||
				outsqlda->sqlvar[i].sqltype==SQL_VARYING+1) {
			outsqlda->sqlvar[i].sqldata=field[i].textbuffer;
			field[i].sqlrtype=VARYING_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_SHORT ||
				outsqlda->sqlvar[i].sqltype==SQL_SHORT+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].shortbuffer;
			field[i].sqlrtype=SHORT_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_LONG || 
				outsqlda->sqlvar[i].sqltype==SQL_LONG+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].longbuffer;
			field[i].sqlrtype=LONG_DATATYPE;
		#ifdef SQL_INT64
		} else if (outsqlda->sqlvar[i].sqltype==SQL_INT64 || 
				outsqlda->sqlvar[i].sqltype==SQL_INT64+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].int64buffer;
			field[i].sqlrtype=INT64_DATATYPE;
		#endif
		} else if (outsqlda->sqlvar[i].sqltype==SQL_FLOAT ||
			outsqlda->sqlvar[i].sqltype==SQL_FLOAT+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].floatbuffer;
			field[i].sqlrtype=FLOAT_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_DOUBLE ||
			outsqlda->sqlvar[i].sqltype==SQL_DOUBLE+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].doublebuffer;
			field[i].sqlrtype=DOUBLE_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_D_FLOAT ||
			outsqlda->sqlvar[i].sqltype==SQL_D_FLOAT+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].doublebuffer;
			field[i].sqlrtype=D_FLOAT_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_ARRAY || 
				outsqlda->sqlvar[i].sqltype==SQL_ARRAY+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].quadbuffer;
			field[i].sqlrtype=ARRAY_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_QUAD || 
				outsqlda->sqlvar[i].sqltype==SQL_QUAD+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].quadbuffer;
			field[i].sqlrtype=QUAD_DATATYPE;
		#ifdef SQL_TIMESTAMP
		} else if (outsqlda->sqlvar[i].sqltype==SQL_TIMESTAMP || 
				outsqlda->sqlvar[i].sqltype==SQL_TIMESTAMP+1) {
		#else
		} else if (outsqlda->sqlvar[i].sqltype==SQL_DATE || 
				outsqlda->sqlvar[i].sqltype==SQL_DATE+1) {
		#endif
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].timestampbuffer;
			field[i].sqlrtype=TIMESTAMP_DATATYPE;
		#ifdef SQL_TIMESTAMP
		} else if (outsqlda->sqlvar[i].sqltype==SQL_TYPE_TIME || 
				outsqlda->sqlvar[i].sqltype==SQL_TYPE_TIME+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].timebuffer;
			field[i].sqlrtype=TIME_DATATYPE;
		} else if (outsqlda->sqlvar[i].sqltype==SQL_TYPE_DATE || 
				outsqlda->sqlvar[i].sqltype==SQL_TYPE_DATE+1) {
			outsqlda->sqlvar[i].sqldata=
					(char *)&field[i].datebuffer;
			field[i].sqlrtype=DATE_DATATYPE;
		#endif
		} else if (outsqlda->sqlvar[i].sqltype==SQL_BLOB || 
				outsqlda->sqlvar[i].sqltype==SQL_BLOB+1) {
			outsqlda->sqlvar[i].sqltype=SQL_BLOB;
			outsqlda->sqlvar[i].sqldata=(char *)NULL;
			field[i].sqlrtype=BLOB_DATATYPE;
		} else {
			outsqlda->sqlvar[i].sqltype=SQL_VARYING;
			outsqlda->sqlvar[i].sqldata=field[i].textbuffer;
			field[i].sqlrtype=UNKNOWN_DATATYPE;
		}
	}

	// execute the query
	if (isc_dsql_execute(error,&tr,&stmt,1,insqlda)) {
		return 0;
	}

	// For "create" queries, we have to commit the creation or subsequent
	// queries in the same transaction can't access the newly created
	// object.  This is kind of strange, possibly a bug, I don't know
	// for sure, but this code should work around it.
	// Hmm, commit's and rollback's seem to require an actual call to
	// isc_commit_transaction and isc_rollback_transaction too...
	if (querytype==isc_info_sql_stmt_ddl || 
			querytype==isc_info_sql_stmt_commit) {
		commit();
	} else if (querytype==isc_info_sql_stmt_rollback) {
		rollback();
	}
	return 1;
}

int	interbaseconnection::queryIsNotSelect() {

	if (querytype==isc_info_sql_stmt_select) {
		return 0;
	}
	return 1;
}

int	interbaseconnection::queryIsCommitOrRollback() {
	if (querytype==isc_info_sql_stmt_commit || 
			querytype==isc_info_sql_stmt_rollback) {
		return 1;
	}
	return 0;
}

void	interbaseconnection::commit() {
	isc_commit_transaction(error,&tr);
	isc_start_transaction(error,&tr,1,&db,0,NULL);
}

void	interbaseconnection::rollback() {
	isc_rollback_transaction(error,&tr);
	isc_start_transaction(error,&tr,1,&db,0,NULL);
}

char	*interbaseconnection::getErrorMessage(int *liveconnection) {

	char	msg[512];
	long	*pvector=error;

	// declare a buffer for the error
	if (errormsg) {
		delete errormsg;
	}
	errormsg=new strstream();

	// get the status message
	while (isc_interprete(msg,&pvector)) {
		*errormsg << msg << " " << endl;
	}

	// get the error message
	long	sqlcode=isc_sqlcode(error);
	isc_sql_interprete(sqlcode, msg, 512);
	*errormsg << msg << ends;

	return errormsg->str();
}

void	interbaseconnection::returnHeader() {

	// send row counts
	sendRowCounts((long)-1,(long)-1);

	// a useful variable
	int	type;


	// for each column...
	for (int i=0; i<outsqlda->sqld; i++) {

		// send column definition
		sendColumnDefinition(outsqlda->sqlvar[i].sqlname,
					strlen(outsqlda->sqlvar[i].sqlname),
					field[i].sqlrtype,
					outsqlda->sqlvar[i].sqllen);
	}
}

int	interbaseconnection::noRowsToReturn() {

	if (outsqlda->sqld==0) {
		return 1;
	}
	return 0;
}

int	interbaseconnection::fetchRow() {

	ISC_STATUS	retcode;
	if (retcode=isc_dsql_fetch(error,&stmt,1,outsqlda)) {
		// if retcode is 100L, then there are no more rows,
		// otherwise, there is an error... how do I handle this?
		return 0;
	}
	return 1;
}

void	interbaseconnection::returnRow() {

	for (int col=0; col<outsqlda->sqld; col++) {

		// handle a null field
		if ((outsqlda->sqlvar[col].sqltype & 1) && 
			field[col].nullindicator==-1) {
			sendNullField();
			continue;
		}

		// handle a non-null field
		if (outsqlda->sqlvar[col].sqltype==SQL_TEXT ||
				outsqlda->sqlvar[col].sqltype==SQL_TEXT+1) {
			// text fields are easy
			sendField(field[col].textbuffer,
					strlen(field[col].textbuffer));
		} else if (outsqlda->sqlvar[col].sqltype==SQL_SHORT ||
				outsqlda->sqlvar[col].sqltype==SQL_SHORT+1) {
			strstream	buffer;
			buffer << field[col].shortbuffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		} else if (outsqlda->sqlvar[col].sqltype==SQL_LONG ||
				outsqlda->sqlvar[col].sqltype==SQL_LONG+1) {
			strstream	buffer;
			buffer << field[col].longbuffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		} else if (outsqlda->sqlvar[col].sqltype==SQL_FLOAT ||
				outsqlda->sqlvar[col].sqltype==SQL_FLOAT+1) {
			strstream	buffer;
			buffer << setprecision(10);
			buffer << field[col].floatbuffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		} else if (outsqlda->sqlvar[col].sqltype==SQL_DOUBLE ||
				outsqlda->sqlvar[col].sqltype==SQL_DOUBLE+1 ||
				outsqlda->sqlvar[col].sqltype==SQL_D_FLOAT ||
				outsqlda->sqlvar[col].sqltype==SQL_D_FLOAT+1) {
			strstream	buffer;
			buffer << setprecision(20);
			buffer << field[col].doublebuffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		} else if (outsqlda->sqlvar[col].sqltype==SQL_VARYING ||
				outsqlda->sqlvar[col].sqltype==SQL_VARYING+1) {
			// the first 2 bytes are the length in 
			// an SQL_VARYING field
			short	size;
			memcpy((void *)&size,(void *)field[col].textbuffer,
					sizeof(short));
			sendField(field[col].textbuffer+sizeof(short),size);
		#ifdef SQL_INT64
		} else if (outsqlda->sqlvar[col].sqltype==SQL_INT64 ||
				outsqlda->sqlvar[col].sqltype==SQL_INT64+1) {
			// int64's are weird.  To the left of the decimal
			// point is the value/10^scale, to the right is
			// value%10^scale
			strstream	buffer;
			if (outsqlda->sqlvar[col].sqlscale) {

				buffer << field[col].int64buffer/(int)pow(10.0,(double)-outsqlda->sqlvar[col].sqlscale) << ".";

				strstream	decimal;
				decimal << field[col].int64buffer%(int)pow(10.0,(double)-outsqlda->sqlvar[col].sqlscale);
			
				// gotta get the right number of decimal places
				for (int i=decimal.rdbuf()->pcount();
					i<-outsqlda->sqlvar[col].sqlscale;
					i++) {
					decimal << "0";
				}
				decimal << ends;
				buffer << decimal.str() << ends;
			} else {
				buffer << field[col].int64buffer << ends;
			}
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		#endif
		} else if (outsqlda->sqlvar[col].sqltype==SQL_ARRAY ||
			outsqlda->sqlvar[col].sqltype==SQL_ARRAY+1 ||
			outsqlda->sqlvar[col].sqltype==SQL_QUAD ||
			outsqlda->sqlvar[col].sqltype==SQL_QUAD+1) {
			// have to handle arrays for real here...
			sendNullField();
		#ifdef SQL_TIMESTAMP
		} else if (outsqlda->sqlvar[col].sqltype==SQL_TIMESTAMP ||
			outsqlda->sqlvar[col].sqltype==SQL_TIMESTAMP+1) {
			// decode the timestamp
			tm	entry_timestamp;
			isc_decode_timestamp(&field[col].timestampbuffer,
							&entry_timestamp);
		#else
		} else if (outsqlda->sqlvar[col].sqltype==SQL_DATE ||
			outsqlda->sqlvar[col].sqltype==SQL_DATE+1) {
			// decode the timestamp
			tm	entry_timestamp;
			isc_decode_date(&field[col].timestampbuffer,
							&entry_timestamp);
		#endif
			// build a string of "yyyy-mm-dd hh:mm:ss" format
			strstream	buffer;
			buffer << entry_timestamp.tm_year+1900;
			buffer << "-";
			buffer << setfill('0') << setw(2);
			buffer << entry_timestamp.tm_mon+1;
			buffer << "-";
			buffer << setfill('0') << setw(2);
			buffer << entry_timestamp.tm_mday;
			buffer << " ";
			buffer << setfill('0') << setw(2);
			buffer << entry_timestamp.tm_hour;
			buffer << ":";
			buffer << setfill('0') << setw(2);
			buffer << entry_timestamp.tm_min;
			buffer << ":";
			buffer << setfill('0') << setw(2);
			buffer << entry_timestamp.tm_sec;
			buffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		#ifdef SQL_TIMESTAMP
		} else if (outsqlda->sqlvar[col].sqltype==SQL_TYPE_TIME ||
			outsqlda->sqlvar[col].sqltype==SQL_TYPE_TIME+1) {
			// decode the time
			tm	entry_time;
			isc_decode_sql_time(&field[col].timebuffer,
							&entry_time);
			// build a string of "hh:mm:ss" format
			strstream	buffer;
			buffer << setfill('0') << setw(2) << entry_time.tm_hour;
			buffer << ":";
			buffer << setfill('0') << setw(2) << entry_time.tm_min;
			buffer << ":";
			buffer << setfill('0') << setw(2) << entry_time.tm_sec;
			buffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		} else if (outsqlda->sqlvar[col].sqltype==SQL_TYPE_DATE ||
			outsqlda->sqlvar[col].sqltype==SQL_TYPE_DATE+1) {
			// decode the date
			tm	entry_date;
			isc_decode_sql_date(&field[col].datebuffer,
							&entry_date);
			// build a string of "yyyy-mm-dd" format
			strstream	buffer;
			buffer << entry_date.tm_year+1900;
			buffer << "-";
			buffer << setfill('0') << setw(2);
			buffer << entry_date.tm_mon+1;
			buffer << "-";
			buffer << setfill('0') << setw(2);
			buffer << entry_date.tm_mday;
			buffer << ends;
			sendField(buffer.str(),buffer.rdbuf()->pcount()-1);
		#endif
		} else if (outsqlda->sqlvar[col].sqltype==SQL_BLOB ||
				outsqlda->sqlvar[col].sqltype==SQL_BLOB+1) {
			// have to handle blobs for real here...
			sendNullField();
		}
	}
}
void	interbaseconnection::cleanUpData() {
	isc_dsql_free_statement(error,&stmt,DSQL_drop);
}

int	interbaseconnection::ping() {
	return 1;
}

char	*interbaseconnection::identify() {
	return "interbase";
}
