// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <lagoconnection.h>

#include <datatypes.h>

#include <stdlib.h>

lagoconnection::lagoconnection() {
	lagocontext=Lnewctx();
}

lagoconnection::~lagoconnection() {
	Ldelctx(lagocontext);
}

int	lagoconnection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	lagoconnection::handleConnectString() {
	host=connectStringValue("host");
	port=connectStringValue("port");
	db=connectStringValue("db");
	user=connectStringValue("user");
	password=connectStringValue("password");
}

int	lagoconnection::logIn() {

	// handle host
	char	*hostval="localhost";
	if (host && host[0]) {
		hostval=host;
	}

	// handle port
	char	*portval="7412";
	if (port && port[0]) {
		portval=port;
	}

	// handle db
	if (!(db && db[0])) {
		cout << "No db was specified in the connect string." << endl;
		return 0;
	}

	// handle user
	if (!(user && user[0])) {
		cout << "No user was specified in the connect string." << endl;
		return 0;
	}

	// handle password
	if (!(password && password[0])) {
		cout << "No password was specified in the connect string.";
		cout << endl;
		return 0;
	}

	if (Lconnect(lagocontext,hostval,portval,db,user,password)==-1) {
		return 0;
	}
	return 1;
}

void	lagoconnection::logOut() {
	Ldisconnect(lagocontext);
}

int	lagoconnection::executeQuery(char *query, long length) {

	// initialize return values
	ncols=0;
	nrows=0;

	// execute the query
	if (!(lagoresult=Lquery(lagocontext,query))) {
		return 0;
	}

	// workaround, lagoresult doesn't necessarily get set to 0 for
	// every error such as a select on a non-existant table
	char	*err=(char *)Lgeterrmsg(lagocontext);
	if (err && err[0]) {
		return 0;
	}

	// get the column count
	ncols=Lgetncols(lagoresult);

	// get the row count
	nrows=Lgetnrows(lagoresult);

	return 1;
}

char	*lagoconnection::getErrorMessage(int *liveconnection) {

	// only return an error message of the error wasn't a dead database
	*liveconnection=Lisconnected(lagocontext);
	if (*liveconnection) {
		return (char *)Lgeterrmsg(lagocontext);
	} else {
		return "";
	}
}

void	lagoconnection::returnHeader() {

	// send row counts (affected row count unknown in Lago)
	sendRowCounts((long)nrows,(long)-1);

	// gonna need this later
	int	length;
	int	type;
	char	*name;

	// for each column...
	for (int i=1; i<ncols+1; i++) {

		// set column type
		LType	coltype=Lgetcoltype(lagoresult,i);
		if (coltype==LSQL_UNDEFINED) {
			type=UNDEFINED_DATATYPE;
		} else if (coltype==LSQL_SMALLINT) {
			type=SMALLINT_DATATYPE;
		} else if (coltype==LSQL_INT) {
			type=INT_DATATYPE;
		} else if (coltype==LSQL_FLOAT) {
			type=FLOAT_DATATYPE;
		} else if (coltype==LSQL_DOUBLE) {
			type=DOUBLE_DATATYPE;
		} else if (coltype==LSQL_CHAR) {
			type=CHAR_DATATYPE;
		} else if (coltype==LSQL_VARCHAR) {
			type=VARCHAR_DATATYPE;
		} else if (coltype==LSQL_DATE) {
			type=DATE_DATATYPE;
		} else if (coltype==LSQL_TIME) {
			type=TIME_DATATYPE;
		} else if (coltype==LSQL_TIMESTAMP) {
			type=TIMESTAMP_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// append ","column size" to the header
		length=Lgetcolprec(lagoresult,i);
		if (Lgetcolscale(lagoresult,i)) {
			// make sure to add 1 for the decimal point
			length=length+Lgetcolprec(lagoresult,i)+1;
		}

		// send the column definition
		name=(char *)Lgetcolname(lagoresult,i);
		sendColumnDefinition(name,strlen(name),type,length);
	}
}

int	lagoconnection::noRowsToReturn() {

	// for queries which returned nothing, such as DML or blank selects,
	// abort early, unfortunately, Lgetnrows doesn't always work correctly,
	// hope that gets fixed some day
	if (!Lgetnrows(lagoresult)) {
		return 1;
	}
	return 0;
}

int	lagoconnection::fetchRow() {
	return Lfetch(lagoresult)!=LFETCH_END;
}

void	lagoconnection::returnRow() {

	// a useful variable
	char	*field;

	// here's that 1 based column thing again
	for (int col=1; col<ncols+1; col++) {

		field=(char *)Lgetasstr(lagoresult,col);
		if (field) {
			sendField(field,strlen(field));
		} else {
			sendNullField();
		}
	}
}

void	lagoconnection::cleanUpData() {
}

int	lagoconnection::ping() {
	// I could use Ltestpkt here but I don't know how...
	return 1;
}

char	*lagoconnection::identify() {
	return "lago";
}

int	lagoconnection::isTransactional() {
	return 0;
}
