// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <msqlconnection.h>

#include <datatypes.h>

msqlconnection::msqlconnection() {
}

msqlconnection::~msqlconnection() {
}

int	msqlconnection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	msqlconnection::handleConnectString() {
	host=connectStringValue("host");
	db=connectStringValue("db");
}

int	msqlconnection::logIn() {

	// handle db
	char	*dbval;
	if (db && db[0]) {
		dbval=db;
	} else {
		dbval="";
	}

	if ((msql=msqlConnect(host))==-1) {
		return 0;
	}

	if (msqlSelectDB(msql,dbval)==-1) {
		logOut();
		return 0;
	} else {
		return 1;
	}
}

void	msqlconnection::logOut() {
	msqlClose(msql);
}

int	msqlconnection::executeQuery(char *query, long length) {

	// initialize return values
	ncols=0;
	nrows=0;

	// execute the query
	if (msqlQuery(msql,query)==-1) {
		return 0;
	}

	// store the result set
	if ((msqlresult=msqlStoreResult())==(m_result *)NULL) {

		// if no error message was generated then the query
		// must have been some DDL or DML, have to check
		// the 0'th array element here, it's kinda weird
		if (msqlErrMsg[0]) {
			return 0;
		} else {
			return 1;
		}
	}

	// get the column count
	ncols=msqlNumFields(msqlresult);

	// get the row count
	nrows=msqlNumRows(msqlresult);

	return 1;
}

char	*msqlconnection::getErrorMessage(int *liveconnection) {

	*liveconnection=1;
	return msqlErrMsg;
}

void	msqlconnection::returnHeader() {

	// send row counts (affected row count unknown in msql)
	sendRowCounts((long)nrows,(long)-1);

	// for dml/ddl queries, return no header
	if (!msqlresult) {
		return;
	}

	// a useful variable
	int	type;

	// position ourselves on the first field
	msqlFieldSeek(msqlresult,0);

	// for each column...
	for (int i=0; i<ncols; i++) {

		// fetch the field
		msqlfield=msqlFetchField(msqlresult);

		// append column type to the header
		if (msqlfield->type==CHAR_TYPE) {
			type=CHAR_DATATYPE;
		} else if (msqlfield->type==TEXT_TYPE) {
			type=TEXT_DATATYPE;
		} else if (msqlfield->type==INT_TYPE) {
			type=INT_DATATYPE;
		} else if (msqlfield->type==UINT_TYPE) {
			type=UINT_DATATYPE;
		} else if (msqlfield->type==MONEY_TYPE) {
			type=MONEY_DATATYPE;
		} else if (msqlfield->type==LAST_REAL_TYPE) {
			type=LASTREAL_DATATYPE;
		} else if (msqlfield->type==REAL_TYPE) {
			type=REAL_DATATYPE;
		} else if (msqlfield->type==DATE_TYPE) {
			type=DATE_DATATYPE;
		} else if (msqlfield->type==TIME_TYPE) {
			type=TIME_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send the column definition
		sendColumnDefinition(msqlfield->name,
					strlen(msqlfield->name),
					type,msqlfield->length);
	}
}

int	msqlconnection::noRowsToReturn() {

	// for dml/ddl queries, return no data
	if (!msqlresult) {
		return 1;
	}
	return 0;
}

int	msqlconnection::fetchRow() {
	return (msqlrow=msqlFetchRow(msqlresult))!=NULL;
}

void	msqlconnection::returnRow() {

	for (int col=0; col<ncols; col++) {

		if (msqlrow[col]) {
			sendField(msqlrow[col],strlen(msqlrow[col]));
		} else {
			sendNullField();
		}
	}
}


void	msqlconnection::cleanUpData() {

	if (msqlresult) {
		msqlFreeResult(msqlresult);
	}
}

int	msqlconnection::ping() {

	// no good way to do this with msql
	return 1;
}

char	*msqlconnection::identify() {
	return "msql";
}

int	msqlconnection::isTransactional() {
	return 0;
}
