// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <oracle7connection.h>

#include <datatypes.h>

#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

ub2	errors[512];

oracle7connection::oracle7connection() {
	errormessage=NULL;
}

oracle7connection::~oracle7connection() {
	delete errormessage;
}

int	oracle7connection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	oracle7connection::handleConnectString() {
	user=connectStringValue("user");
	password=connectStringValue("password");
	sid=connectStringValue("oracle_sid");
	home=connectStringValue("oracle_home");
}

int	oracle7connection::logIn() {

	// handle ORACLE_HOME
	if (home) {
		setEnv("ORACLE_HOME",home);
	} else {
		if (!getenv("ORACLE_HOME")) {
			cerr << "No ORACLE_HOME environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// handle ORACLE_SID
	if (sid) {
		setEnv("ORACLE_SID",sid);
	} else {
		if (!getenv("ORACLE_SID")) {
			cerr << "No ORACLE_SID environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// handle TWO_TASK
	if (sid) {
		setEnv("TWO_TASK",sid);
	} else {
		if (!getenv("TWO_TASK")) {
			cerr << "No TWO_TASK environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// log in
	if (!olog(&lda,(ub1 *)hda,
			(text *)user,-1,(text *)password,-1,(text *)0,-1,
			(ub4)OCI_LM_DEF)) {
		// open a cursor
		if (!oopen(&cda,&lda,(text *)0,-1,-1,(text *)0,-1)) {
			return 1;
		} else {
			logOut();
			return 0;
		}

	} else {
		return 0;
	}
}

void	oracle7connection::logOut() {
	oclose(&cda);
	ologof(&lda);
}

int	oracle7connection::prepareQuery(char *query, long length) {

	// parse the query
	if (oparse(&cda,(text *)query,(sb4)length,
		(sword)PARSE_NO_DEFER,(ub4)PARSE_V7_LNG)) {
		return 0;
	}
	return 1;
}

int	oracle7connection::inputBindValue(char *variable, 
						short variablesize,
						char *value, 
						short valuesize,
						short *isnull) {

	// bind the value to the variable
	if (obndrv(&cda,(text *)variable,(sword)variablesize,
			(ub1 *)value,(sword)valuesize+1,NULL_TERMINATED_STRING,
			-1,(sb2 *)isnull,(text *)0,-1,-1)) {
		return 0;
	}
	return 1;
}

int	oracle7connection::inputBindValue(char *variable, 
						short variablesize,
						long *value) {
	
	// bind the value to the variable
	if (obndrv(&cda,(text *)variable,(sword)variablesize,
			(ub1 *)value,(sword)sizeof(long),LONG_BIND_TYPE,
			-1,(sb2 *)0,(text *)0,-1,-1)) {
		return 0;
	}
	return 1;
}

int	oracle7connection::inputBindValue(char *variable, 
						short variablesize,
						double *value,
						short precision,
						short scale) {
	
	// bind the value to the variable
	if (obndrv(&cda,(text *)variable,(sword)variablesize,
			(ub1 *)value,(sword)sizeof(double),DOUBLE_BIND_TYPE,
			(sword)scale,(sb2 *)0,(text *)0,-1,-1)) {
		return 0;
	}
	return 1;
}

int	oracle7connection::outputBindValue(char *variable, short variablesize,
				char *value, short valuesize, short *isnull) {

	// bind the value to the variable
	if (obndrv(&cda,(text *)variable,(sword)variablesize,
			(ub1 *)value,(sword)valuesize,
			NULL_TERMINATED_STRING,
			-1,(sb2 *)isnull,(text *)0,-1,-1)) {
		return 0;
	}
	return 1;
}

short	oracle7connection::nonNullBindValue() {
	return 0;
}

short	oracle7connection::nullBindValue() {
	return -1;
}

int	oracle7connection::bindValueIsNull(short isnull) {
	if (isnull<0) {
		return 1;
	}
	return 0;
}

int	oracle7connection::executeQuery(char *query, long length) {

	// initialize the column count
	ncols=0;

	// if the query is a select, describe/define it
	if (cda.ft==SELECT_QUERY) {

		// run through the columns...
		while (1) {

			// init the column name size
			desc[ncols].buflen=MAX_ITEM_BUFFER_SIZE;

			// describe the column
			if (!odescr(&cda,ncols+1,
				&desc[ncols].dbsize, &desc[ncols].dbtype,
				&desc[ncols].buf[0], &desc[ncols].buflen,
				&desc[ncols].dsize, &desc[ncols].precision,
				&desc[ncols].scale, &desc[ncols].nullok)) {

				// define the column
				if (odefin(&cda,ncols+1,
						def[ncols].buf,
						MAX_ITEM_BUFFER_SIZE,
						NULL_TERMINATED_STRING,
						-1,
						&def[ncols].indp,
						(text *)0,
						-1,
						-1,
						&def[ncols].col_retlen,
						&def[ncols].col_retcode)) {
					return 0;
				}

			} else {

				// if we're at the end of the column list...
				if (cda.rc==NOT_IN_LIST) {
					break;
				} else {
					return 0;
				}
			}
	
			ncols++;
		}
	}

	// execute the query
	if (oexec(&cda)) {
		return 0;
	}

	return 1;
}

int	oracle7connection::queryIsNotSelect() {

	if (cda.ft!=SELECT_QUERY) {
		return 1;
	}
	return 0;
}

int	oracle7connection::queryIsCommitOrRollback() {

	if (cda.ft==COMMIT_QUERY || cda.ft==ROLLBACK_QUERY) {
		return 1;
	}
	return 0;
}

void	oracle7connection::commit() {
	ocom(&lda);
}

void	oracle7connection::rollback() {
	orol(&lda);
}

char	*oracle7connection::getErrorMessage(int *liveconnection) {

	// get the message from oracle
	text	message[512];
	sword	n=oerhms(&lda,cda.rc,message,(sword)sizeof(message));

	// check for dead connection
	if (!strncmp((char *)message,"ORA-03144",9) || 
		!strncmp((char *)message,"ORA-03113",9)) {
		*liveconnection=0;
	}

	// only return an error message if the error wasn't a dead database
	if (errormessage) {
		delete errormessage;
	}
	errormessage=new strstream();
	if (*liveconnection) {
		for (int i=0; i<n; i++) {
			*errormessage << message[i];
		}
	}
	*errormessage << ends;

	return errormessage->str();
}

void	oracle7connection::returnHeader() {

	// send row counts (actual row count unknown in oracle)
	sendRowCounts((long)-1,(long)cda.rpc);

	// a useful variables
	int		type;

	// for each column...
	for (int i=0; i<ncols; i++) {

		// set column type
		if (desc[i].dbtype==VARCHAR2_TYPE) {
			type=VARCHAR2_DATATYPE;
		} else if (desc[i].dbtype==NUMBER_TYPE) {
			type=NUMBER_DATATYPE;
		} else if (desc[i].dbtype==LONG_TYPE) {
			type=LONG_DATATYPE;
		} else if (desc[i].dbtype==ROWID_TYPE) {
			type=ROWID_DATATYPE;
		} else if (desc[i].dbtype==DATE_TYPE) {
			type=DATE_DATATYPE;
		} else if (desc[i].dbtype==RAW_TYPE) {
			type=RAW_DATATYPE;
		} else if (desc[i].dbtype==LONG_RAW_TYPE) {
			type=LONG_RAW_DATATYPE;
		} else if (desc[i].dbtype==CHAR_TYPE) {
			type=CHAR_DATATYPE;
		} else if (desc[i].dbtype==MLSLABEL_TYPE) {
			type=MLSLABEL_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send the column definition
		sendColumnDefinition((char *)desc[i].buf,
					(short)desc[i].buflen,
					type,
					(int)desc[i].dbsize);
	}
}

int	oracle7connection::noRowsToReturn() {
	if (cda.ft!=SELECT_QUERY) {
		return 1;
	}
	return 0;
}

int	oracle7connection::fetchRow() {
	return (!ofetch(&cda) && (&cda)->rc!=NO_DATA_FOUND);
}

void	oracle7connection::returnRow() {

	for (sword col=0; col<ncols; col++) {

		// handle NULL's
		if (def[col].indp==-1) {
			sendNullField();
			continue;
		}

		// handle long datatypes
		if (desc[col].dbtype==LONG_TYPE ||
			desc[col].dbtype==LONG_RAW_TYPE) {

			ub4	retlen=MAX_ITEM_BUFFER_SIZE;
			sb4	offset=0;

			startSendingLong();

			while (retlen==MAX_ITEM_BUFFER_SIZE) {

				oflng(&cda,col+1,def[col].buf,
					MAX_ITEM_BUFFER_SIZE,1,
					&retlen,offset);

				sendLongSegment((char *)def[col].buf,
						(long)retlen);

				offset=offset+retlen;
			}

			endSendingLong();

			continue;

		}

		// handle normal datatypes
		sendField((char *)def[col].buf,(int)def[col].col_retlen);
	}
}

void	oracle7connection::cleanUpData() {
	ocan(&cda);
}

int	oracle7connection::ping() {

	if (!executeQuery("select 1 from dual",18)) {
		int	liveconnection;
		getErrorMessage(&liveconnection);
		if (liveconnection) {
			return 1;
		}
		return 0;
	}
	return 1;
}

char	*oracle7connection::identify() {
	return "oracle7";
}
