// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#ifndef ORACLECONNECTION_H
#define ORACLECONNECTION_H

#define NUM_CONNECT_STRING_VARS 3

#include <connection.h>

#include <strstream.h>
#include <fstream.h>

extern "C" {
	#include <oratypes.h>
	#include <ocidfn.h>
	#include <ociapr.h>

	#define MAX_ITEM_BUFFER_SIZE	4096
	#define MAX_SELECT_LIST_SIZE	256

	#define PARSE_NO_DEFER		0
	#define PARSE_V7_LNG		2

	#define SELECT_QUERY 4
	#define COMMIT_QUERY 54
	#define ROLLBACK_QUERY 55

	#define NOT_IN_LIST 1007
	#define NO_DATA_FOUND 1403

	#define VARCHAR2_TYPE 1
	#define	NUMBER_TYPE 2
	#define	LONG_TYPE 8
	#define ROWID_TYPE 11
	#define DATE_TYPE 12
	#define RAW_TYPE 23
	#define LONG_RAW_TYPE 24
	#define CHAR_TYPE 96
	#define MLSLABEL_TYPE 105

	#define VARCHAR2_BIND_TYPE 1
	#define LONG_BIND_TYPE 3
	#define DOUBLE_BIND_TYPE 4
	#define NULL_TERMINATED_STRING 5
}

struct describe {
	sb4	dbsize;
	sb2	dbtype;
	sb1	buf[MAX_ITEM_BUFFER_SIZE];
	sb4	buflen;
	sb4	dsize;
	sb2	precision;
	sb2	scale;
	sb2	nullok;
};
	
struct define {
	ub1	buf[MAX_ITEM_BUFFER_SIZE];
	sb2	indp;
	ub2	col_retlen;
	ub2	col_retcode;
};

class oracle7connection : public connection {
	public:
			oracle7connection();
			~oracle7connection();
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		void	logOut();
		int	prepareQuery(char *query, long length);
		int	inputBindValue(char *variable, 
					short variablesize,
					char *value, 
					short valuesize,
					short *isnull);
		int	inputBindValue(char *variable, 
					short variablesize,
					long *value);
		int	inputBindValue(char *variable, 
					short variablesize,
					double *value,
					short precision,
					short scale);
		int	outputBindValue(char *variable, 
					short variablesize,
					char *value,
					short valuesize,
					short *isnull);
		short	nonNullBindValue();
		short	nullBindValue();
		int	bindValueIsNull(short isnull);
		int	executeQuery(char *query, long length);
		int	queryIsNotSelect();
		int	queryIsCommitOrRollback();
		void	commit();
		void	rollback();
		char	*getErrorMessage(int *liveconnection);
		void	returnHeader();
		int	noRowsToReturn();
		int	fetchRow();
		void	returnRow();
		void	cleanUpData();
		int	ping();
		char	*identify();

		Lda_Def		lda;
		Cda_Def		cda;
		ub4		hda[256/sizeof(ub4)];
		sword		ncols;
		strstream	*errormessage;

		describe	desc[MAX_SELECT_LIST_SIZE];
		define		def[MAX_SELECT_LIST_SIZE];

		char		*user;
		char		*password;
		char		*home;
		char		*sid;
};

#endif
