// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <oracle8connection.h>

#include <datatypes.h>

#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
	#include <strings.h>
#endif

#ifdef __GNUC__
static	sb2	oracle8connection::null_ind;
#endif

oracle8connection::oracle8connection() {
	errormessage=NULL;

	null_ind=-1;

	// init OCI
	OCIInitialize(OCI_DEFAULT,(dvoid *)0, 
			(dvoid*(*)(dvoid *,size_t))0,
			(dvoid*(*)(dvoid *,dvoid *,size_t))0, 
			(void(*)(dvoid *,dvoid *))0);

	// init the environment
	OCIEnvInit((OCIEnv **)&env,OCI_DEFAULT,(size_t)0,(dvoid **)0); 

	// allocate an error handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&err,OCI_HTYPE_ERROR,
				(size_t)0,(dvoid **)0);
	// allocate a server handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&srv,OCI_HTYPE_SERVER,
				(size_t)0,(dvoid **)0);
	// allocate a service context handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&svc,OCI_HTYPE_SVCCTX,
				(size_t)0,(dvoid **)0);

}

oracle8connection::~oracle8connection() {

	// free the service, server and error handles
	OCIHandleFree(svc,OCI_HTYPE_SVCCTX);
	OCIHandleFree(srv,OCI_HTYPE_SERVER);
	OCIHandleFree(err,OCI_HTYPE_ERROR);

	delete errormessage;
}

int	oracle8connection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	oracle8connection::handleConnectString() {
	user=connectStringValue("user");
	password=connectStringValue("password");
	sid=connectStringValue("oracle_sid");
	home=connectStringValue("oracle_home");
}

int	oracle8connection::logIn() {

	// handle ORACLE_HOME
	if (home) {
		setEnv("ORACLE_HOME",home);
	} else {
		if (!getenv("ORACLE_HOME")) {
			cerr << "No ORACLE_HOME environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// handle ORACLE_SID
	if (sid) {
		setEnv("ORACLE_SID",sid);
	} else {
		if (!getenv("ORACLE_SID")) {
			cerr << "No ORACLE_SID environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// handle TWO_TASK
	if (sid) {
		setEnv("TWO_TASK",sid);
	} else {
		if (!getenv("TWO_TASK")) {
			cerr << "No TWO_TASK environment variable set or specified in connect string." << endl;
			return 0;
		}
	}

	// attach to the server
	if (OCIServerAttach(srv,err,(text *)sid,strlen(sid),0)!=OCI_SUCCESS) {
		cerr << "Couldn't attach to server." << endl;
		return 0;
	}

	// attach the server to the service
	OCIAttrSet((dvoid *)svc,OCI_HTYPE_SVCCTX,
				(dvoid *)srv,(ub4)0,
				OCI_ATTR_SERVER,(OCIError *)err);

	// allocate a session handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&session,(ub4)OCI_HTYPE_SESSION,
				(size_t)0,(dvoid **)0);

	// set username and password
	OCIAttrSet((dvoid *)session,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)user,(ub4)strlen(user),
				(ub4)OCI_ATTR_USERNAME,err);
	OCIAttrSet((dvoid *)session,(ub4)OCI_HTYPE_SESSION,
				(dvoid *)password,(ub4)strlen(password),
				(ub4)OCI_ATTR_PASSWORD,err);

	// start a session
	if (OCISessionBegin(svc,err,session,
			OCI_CRED_RDBMS,(ub4)OCI_DEFAULT)!=OCI_SUCCESS) {
		cerr << "Couldn't start a session." << endl;
		return 0;
	}

	// attach the session to the service
	OCIAttrSet((dvoid *)svc,(ub4)OCI_HTYPE_SVCCTX,
				(dvoid *)session,(ub4)0,
				(ub4)OCI_ATTR_SESSION,err);

	// allocate a transaction handle
	OCIHandleAlloc((dvoid *)env,(dvoid **)&trans,OCI_HTYPE_TRANS,0,0);

	// attach the transaction to the service
	OCIAttrSet((dvoid *)svc,OCI_HTYPE_SVCCTX,
				(dvoid *)trans,(ub4)0,
				(ub4)OCI_ATTR_TRANS,err);

	return 1;
}

void	oracle8connection::logOut() {

	OCIHandleFree(trans,OCI_HTYPE_TRANS);
	OCISessionEnd(svc,err,session,OCI_DEFAULT);
	OCIHandleFree(session,OCI_HTYPE_SESSION);
	OCIServerDetach(srv,err,OCI_DEFAULT);
}

int	oracle8connection::prepareQuery(char *query, long length) {

	// allocate a statement handle
	if (OCIHandleAlloc((dvoid *)env,(dvoid **)&stmt,OCI_HTYPE_STMT,
					(size_t)0,(dvoid **)0)!=OCI_SUCCESS) {
		return 0;
	}

	// prepare the query
	if (OCIStmtPrepare(stmt,err,(text *)query,(ub4)length,
			(ub4)OCI_NTV_SYNTAX,(ub4)OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}

	return 1;
}

int	oracle8connection::inputBindValue(char *variable, 
						short variablesize,
						char *value, 
						short valuesize,
						short *isnull) {

	OCIBind	*bindpp=NULL;

	// the size of the value must include the terminating NULL
	if (OCIBindByName(stmt,&bindpp,err,
			(text *)variable,(sb4)variablesize,
			(dvoid *)value,(sb4)valuesize+1,
			NULL_TERMINATED_STRING,
			(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
			OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}


int	oracle8connection::inputBindValue(char *variable, 
						short variablesize,
						long *value) {

	OCIBind	*bindpp=NULL;

	if (OCIBindByName(stmt,&bindpp,err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,(sb4)sizeof(long),
				LONG_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}


int	oracle8connection::inputBindValue(char *variable, 
						short variablesize,
						double *value,
						short precision,
						short scale) {

	OCIBind	*bindpp=NULL;

	if (OCIBindByName(stmt,&bindpp,err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,(sb4)sizeof(double),
				DOUBLE_BIND_TYPE,
				(dvoid *)0,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}

int	oracle8connection::outputBindValue(char *variable, 
						short variablesize,
						char *value,
						short valuesize, 
						short *isnull) {

	OCIBind	*bindpp=NULL;
	if (OCIBindByName(stmt,&bindpp,err,
				(text *)variable,(sb4)variablesize,
				(dvoid *)value,
				(sb4)valuesize,
				NULL_TERMINATED_STRING,
				(dvoid *)isnull,(ub2 *)0,(ub2 *)0,0,(ub4 *)0,
				OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}
	return 1;
}

short	oracle8connection::nonNullBindValue() {
	return 0;
}

short	oracle8connection::nullBindValue() {
	return -1;
}

int	oracle8connection::bindValueIsNull(short isnull) {
	if (isnull<0) {
		return 1;
	}
	return 0;
}

int	oracle8connection::executeQuery(char *query, long length) {

	// initialize the column count
	ncols=0;

	// get the type of the query (select, insert, update, etc...)
	if (OCIAttrGet(stmt,OCI_HTYPE_STMT,
			(dvoid *)&stmttype,(ub4 *)0,
			OCI_ATTR_STMT_TYPE,err)!=OCI_SUCCESS) {
		return 0;
	}

	// set up how many times to iterate; 0 for selects, 1 for non-selects
	ub4	iters=1;
	if (stmttype==OCI_STMT_SELECT) {
		iters=0;
	}

	// execute the query
	if (OCIStmtExecute(svc,stmt,err,iters,(ub4)0,NULL,NULL,
					OCI_DEFAULT)!=OCI_SUCCESS) {
		return 0;
	}

	// if the query is a select, describe/define it
	if (stmttype==OCI_STMT_SELECT) {

		// get the column count
		if (OCIAttrGet((dvoid *)stmt,OCI_HTYPE_STMT,
				(dvoid *)&ncols,(ub4 *)0,
				OCI_ATTR_PARAM_COUNT,err)!=OCI_SUCCESS) {
			return 0;
		}

		// run through the columns...
		for (int i=0; i<ncols; i++) {

			// get the entire column definition
			if (OCIParamGet(stmt,OCI_HTYPE_STMT,err,
				(dvoid **)&desc[i].paramd,i+1)!=OCI_SUCCESS) {
				return 0;
			}

			// get the column name
			if (OCIAttrGet((dvoid *)desc[i].paramd,OCI_DTYPE_PARAM,
				(dvoid **)&desc[i].buf,
				(ub4 *)&desc[i].buflen,
				(ub4)OCI_ATTR_NAME,err)!=OCI_SUCCESS) {
				return 0;
			}

			// get the column type
			if (OCIAttrGet((dvoid *)desc[i].paramd,OCI_DTYPE_PARAM,
				(dvoid *)&desc[i].dbtype,(ub4 *)0,
				(ub4)OCI_ATTR_DATA_TYPE,err)!=OCI_SUCCESS) {
				return 0;
			}

			// is the column a LOB?
			if (desc[i].dbtype==BLOB_TYPE ||
				desc[i].dbtype==CLOB_TYPE ||
				desc[i].dbtype==BFILE_TYPE) {

				// return 0 for the size of lobs
				desc[i].dbsize=0;

				// allocate a lob descriptor
				if (OCIDescriptorAlloc((void *)env,
							(void **)&def[i].lob,
							OCI_DTYPE_LOB,0,0)) {
					return 0;
				}

				// define the column as a lob
				if (OCIDefineByPos(stmt,&def[i].def,err,
					i+1,
					(dvoid *)&def[i].lob,
					(sb4)-1,
					desc[i].dbtype,
					(dvoid *)0,
					0,
					(ub2 *)0,
					OCI_DEFAULT)!=OCI_SUCCESS) {
					return 0;
				}

			} else {

				// get the column size
				if (OCIAttrGet((dvoid *)desc[i].paramd,
					OCI_DTYPE_PARAM,
					(dvoid *)&desc[i].dbsize,(ub4 *)0,
					(ub4)OCI_ATTR_DATA_SIZE,err)
					!=OCI_SUCCESS) {
					return 0;
				}

				// if the column is not a LOB, define it,
				// translated to a NULL terminated string
				if (OCIDefineByPos(stmt,&def[i].def,err,
					i+1,
					(dvoid *)&def[i].buf,
					(sb4)MAX_ITEM_BUFFER_SIZE,
					SQLT_STR,
					(dvoid *)&def[i].indp,
					&def[i].col_retlen,
					&def[i].col_retcode,
					OCI_DEFAULT)!=OCI_SUCCESS) {
					return 0;
				}

				// set the lob member to NULL
				def[i].lob=NULL;
			}
		}
	}

	return 1;
}

int	oracle8connection::queryIsNotSelect() {

	if (stmttype==OCI_STMT_SELECT) {
		return 1;
	}
	return 0;
}

int	oracle8connection::queryIsCommitOrRollback() {

	// apparantly in OCI8, the statement type gets 
	// set to 0 for both commits and rollbacks
	if (stmttype==0) {
		return 1;
	}
	return 0;
}

void	oracle8connection::commit() {
	OCITransCommit(svc,err,OCI_DEFAULT);
}

void	oracle8connection::rollback() {
	OCITransRollback(svc,err,OCI_DEFAULT);
}

char	*oracle8connection::getErrorMessage(int *liveconnection) {

	// get the message from oracle
	text	message[512];
	for (int i=0; i<512; i++) {
		message[i]=(char)NULL;
	}
	sb4	errcode;
	OCIErrorGet((dvoid *)err,1,
			(text *)0,&errcode,
			message,sizeof(message),
			OCI_HTYPE_ERROR);
	message[511]=(char)NULL;

	// check for dead connection
	if (errcode==3144 || errcode==3113) {
		*liveconnection=0;
	}

	// only return an error message if the error wasn't a dead database
	if (errormessage) {
		delete errormessage;
	}
	errormessage=new strstream();
	if (*liveconnection) {
		*errormessage << message;
	}
	*errormessage << ends;

	return errormessage->str();
}

void	oracle8connection::returnHeader() {

	// get the affected row count
	ub4	affectedrows;
	OCIAttrGet(stmt,OCI_HTYPE_STMT,
			(dvoid *)&affectedrows,(ub4 *)0,
			OCI_ATTR_ROW_COUNT,err);

	// send row counts (actual row count unknown in oracle)
	// don't know how to get affected row count in OCI8
	sendRowCounts((long)-1,(long)affectedrows);

	// a useful variable
	int	type;

	// for each column...
	for (int i=0; i<ncols; i++) {

		// set column type
		if (desc[i].dbtype==VARCHAR2_TYPE) {
			type=VARCHAR2_DATATYPE;
		} else if (desc[i].dbtype==NUMBER_TYPE) {
			type=NUMBER_DATATYPE;
		} else if (desc[i].dbtype==LONG_TYPE) {
			type=LONG_DATATYPE;
		} else if (desc[i].dbtype==ROWID_TYPE) {
			type=ROWID_DATATYPE;
		} else if (desc[i].dbtype==DATE_TYPE) {
			type=DATE_DATATYPE;
		} else if (desc[i].dbtype==RAW_TYPE) {
			type=RAW_DATATYPE;
		} else if (desc[i].dbtype==LONG_RAW_TYPE) {
			type=LONG_RAW_DATATYPE;
		} else if (desc[i].dbtype==CHAR_TYPE) {
			type=CHAR_DATATYPE;
		} else if (desc[i].dbtype==MLSLABEL_TYPE) {
			type=MLSLABEL_DATATYPE;
		} else if (desc[i].dbtype==BLOB_TYPE) {
			type=BLOB_DATATYPE;
		} else if (desc[i].dbtype==CLOB_TYPE) {
			type=CLOB_DATATYPE;
		} else if (desc[i].dbtype==BFILE_TYPE) {
			type=BFILE_DATATYPE;
		} else {
			type=UNKNOWN_DATATYPE;
		}

		// send the column definition
		sendColumnDefinition((char *)desc[i].buf,
					(short)desc[i].buflen,
					type,
					(int)desc[i].dbsize);
	}
}

int	oracle8connection::noRowsToReturn() {
	if (stmttype!=OCI_STMT_SELECT) {
		return 1;
	}
	return 0;
}

int	oracle8connection::fetchRow() {
	if (OCIStmtFetch(stmt,err,1,OCI_FETCH_NEXT,OCI_DEFAULT)==OCI_SUCCESS) {
		return 1;
	}
	return 0;
}

void	oracle8connection::returnRow() {

	for (sword col=0; col<ncols; col++) {

		// handle NULL's
		if (def[col].indp) {
			sendNullField();
			continue;
		}

		// in OCI8, longs are just like other datatypes, but LOBS
		// are different
		if (desc[col].dbtype==BLOB_TYPE ||
			desc[col].dbtype==CLOB_TYPE ||
			desc[col].dbtype==BFILE_TYPE) {

			// handle lob datatypes
			ub4	retlen=MAX_ITEM_BUFFER_SIZE;
			sb4	offset=1;

			startSendingLong();

			sword	retval=OCI_NEED_DATA;
			while (retval==OCI_NEED_DATA) {

				// read a segment from the lob
				retval=OCILobRead(svc,err,
						def[col].lob,
						&retlen,
						offset,
						(dvoid *)&def[col].buf,
						MAX_ITEM_BUFFER_SIZE,
						(dvoid *)0,
				(sb4 (*)(dvoid *, CONST dvoid *, ub4,ub1))0,
						(ub2)0,
						(ub1)SQLCS_IMPLICIT);

				// send the lob segment
				sendLongSegment((char *)def[col].buf,
						(long)retlen);

				offset=offset+retlen;
			}

			endSendingLong();

			continue;

		}

		// handle normal datatypes
		sendField((char *)def[col].buf,(int)def[col].col_retlen);
	}
}

void	oracle8connection::cleanUpData() {

	// OCI8 version of ocan()
	OCIStmtFetch(stmt,err,0,OCI_FETCH_NEXT,OCI_DEFAULT);

	// free resources
	OCIHandleFree(stmt,OCI_HTYPE_STMT);
	for (int i=0; i<ncols; i++) {
		if (def[i].lob) {
			OCIHandleFree(def[i].lob,OCI_DTYPE_LOB);
		}
		if (def[i].def) {
			OCIHandleFree(def[i].def,OCI_HTYPE_DEFINE);
		}
	}
}

int	oracle8connection::ping() {

	if (!executeQuery("select 1 from dual",18)) {
		int	liveconnection;
		getErrorMessage(&liveconnection);
		if (liveconnection) {
			return 1;
		}
		return 0;
	}
	return 1;
}

char	*oracle8connection::identify() {
	return "oracle8";
}
