// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#ifndef ORACLECONNECTION_H
#define ORACLECONNECTION_H

#define NUM_CONNECT_STRING_VARS 3

#include <connection.h>

#include <strstream.h>
#include <fstream.h>

extern "C" {
	#include <oci.h>

	#define MAX_ITEM_BUFFER_SIZE	4096
	#define MAX_SELECT_LIST_SIZE	256

	#define VARCHAR2_TYPE 1
	#define	NUMBER_TYPE 2
	#define	LONG_TYPE 8
	#define ROWID_TYPE 11
	#define DATE_TYPE 12
	#define RAW_TYPE 23
	#define LONG_RAW_TYPE 24
	#define CHAR_TYPE 96
	#define MLSLABEL_TYPE 105
	#define CLOB_TYPE 112
	#define BLOB_TYPE 113
	#define BFILE_TYPE 114

	#define LONG_BIND_TYPE 3
	#define DOUBLE_BIND_TYPE 4
	#define NULL_TERMINATED_STRING 5
}

struct describe {
	OCIParam	*paramd;
	sb4		dbsize;
	sb2		dbtype;
	text		*buf;
	sb4		buflen;
};
	
struct define {
	OCIDefine	*def;
	OCILobLocator	*lob;
	ub1		buf[MAX_ITEM_BUFFER_SIZE];
	sb2		indp;
	ub2		col_retlen;
	ub2		col_retcode;
};

class oracle8connection : public connection {
	public:
			oracle8connection();
			~oracle8connection();
	private:
		int	getNumberOfConnectStringVars();
		void	handleConnectString();
		int	logIn();
		void	logOut();
		int	prepareQuery(char *query, long length);
		int	inputBindValue(char *variable, 
					short variablesize,
					char *value, 
					short valuesize,
					short *isnull);
		int	inputBindValue(char *variable, 
					short variablesize,
					long *value);
		int	inputBindValue(char *variable, 
					short variablesize,
					double *value,
					short precision,
					short scale);
		int	outputBindValue(char *variable, 
					short variablesize,
					char *value,
					short valuesize,
					short *isnull);
		short	nonNullBindValue();
		short	nullBindValue();
		int	bindValueIsNull(short isnull);
		int	executeQuery(char *query, long length);
		int	queryIsNotSelect();
		int	queryIsCommitOrRollback();
		void	commit();
		void	rollback();
		char	*getErrorMessage(int *liveconnection);
		void	returnHeader();
		int	noRowsToReturn();
		int	fetchRow();
		void	returnRow();
		void	cleanUpData();
		int	ping();
		char	*identify();

		OCIEnv		*env;
		OCIServer	*srv;
		OCIError	*err;
		OCISvcCtx	*svc;
		OCISession	*session;
		OCITrans	*trans;
		OCIStmt		*stmt;
		ub2		stmttype;
		sword		ncols;
		strstream	*errormessage;

		describe	desc[MAX_SELECT_LIST_SIZE];
		define		def[MAX_SELECT_LIST_SIZE];

		char		*user;
		char		*password;
		char		*home;
		char		*sid;

		static	sb2	null_ind;
};

#endif
