// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <postgresqlconnection.h>

#include <datatypes.h>

postgresqlconnection::postgresqlconnection() {
}

postgresqlconnection::~postgresqlconnection() {
}

int	postgresqlconnection::getNumberOfConnectStringVars() {
	return NUM_CONNECT_STRING_VARS;
}

void	postgresqlconnection::handleConnectString() {
	host=connectStringValue("host");
	port=connectStringValue("port");
	options=connectStringValue("options");
	tty=connectStringValue("tty");
	db=connectStringValue("db");
	user=connectStringValue("user");
	password=connectStringValue("password");
}

int	postgresqlconnection::logIn() {

	// log in
	pgconn=PQsetdbLogin(host,port,options,tty,db,user,password);

	// check the status of the login
	if (PQstatus(pgconn)==CONNECTION_BAD) {
		logOut();
		return 0;
	} else {
		return 1;
	}
}

void	postgresqlconnection::logOut() {
	PQfinish(pgconn);
}

int	postgresqlconnection::executeQuery(char *query, long length) {

	// initialize the counts
	ncols=0;
	nrows=0;
	currentrow=-1;

	// execute the query
	if ((pgresult=PQexec(pgconn,query))==(PGresult *)NULL) {
		return 0;
	}

	// handle errors
	ExecStatusType	pgstatus=PQresultStatus(pgresult);
	if (pgstatus==PGRES_BAD_RESPONSE || pgstatus==PGRES_NONFATAL_ERROR ||
		pgstatus==PGRES_FATAL_ERROR) {
		return 0;
	}

	// get the col count
	ncols=PQnfields(pgresult);

	// get the row count
	nrows=PQntuples(pgresult);

	return 1;
}

char	*postgresqlconnection::getErrorMessage(int *liveconnection) {
	*liveconnection=1;
	return PQerrorMessage(pgconn);
}

void	postgresqlconnection::returnHeader() {

	// send row counts (affected rows unknown in postgresql)
	sendRowCounts((long)nrows,(long)-1);

	// some useful variables
	Oid	pgfieldtype;
	int	type;
	char	*name;

	// for each column...
	for (int i=0; i<ncols; i++) {


		// append column type to the header
		// This is strange with POSTGRESQL, there are no actual types,
		// only internal numbers that correspond to types which
		// are defined in a database table.  
		// Some correspond to standard types, but for the rest,
		// I'll just return the number instead of UNKNOWN because
		// it might be useful.
		pgfieldtype=PQftype(pgresult,i);
		if ((int)pgfieldtype==23) {
			type=INT_DATATYPE;
		} else if ((int)pgfieldtype==701) {
			type=FLOAT_DATATYPE;
		} else if ((int)pgfieldtype==21) {
			type=SMALLINT_DATATYPE;
		} else if ((int)pgfieldtype==1042) {
			type=CHAR_DATATYPE;
		} else if ((int)pgfieldtype==1043) {
			type=VARCHAR_DATATYPE;
		} else if ((int)pgfieldtype==1082) {
			type=DATE_DATATYPE;
		} else if ((int)pgfieldtype==1083) {
			type=TIME_DATATYPE;
		} else if ((int)pgfieldtype==1296) {
			type=TIMESTAMP_DATATYPE;
		} else {
			type=-1*(int)PQftype(pgresult,i);
		}

		// send column definition
		name=PQfname(pgresult,i);
		sendColumnDefinition(name,strlen(name),
					type,PQfsize(pgresult,i));
	}
}

int	postgresqlconnection::noRowsToReturn() {

	if (!nrows) {
		return 1;
	}
	return 0;
}

int	postgresqlconnection::fetchRow() {
	currentrow++;
	if (currentrow==nrows) {
		return 0;
	}
	return 1;
}

void	postgresqlconnection::returnRow() {

	char	*field;

	// send the row back
	for (int col=0; col<ncols; col++) {

		// get the row
		field=PQgetvalue(pgresult,currentrow,col);
		if (field) {
			sendField(field,strlen(field));
		} else {
			sendNullField();
		}
	}
}


void	postgresqlconnection::cleanUpData() {
	if (pgresult) {
		PQclear(pgresult);
	}
}

int	postgresqlconnection::ping() {
	if (PQstatus(pgconn)==CONNECTION_OK) {
		return 1;
	}
	return 0;
}

char	*postgresqlconnection::identify() {
	return "postgresql";
}
