// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#include <configfile.h>
#include <iostream.h>

#include <stdlib.h>
#include <string.h>

#ifdef __GNUC__
static	char		*configfile::id;
static	strstream	configfile::connectstring;
static	int		configfile::foundinstance;
static	int		configfile::done;

static	int		configfile::port;
static	char		*configfile::unixport;
static	char		*configfile::dbase;
static	int		configfile::connections;
static	int		configfile::maxconnections;
static	int		configfile::maxqueuelength;
static	int		configfile::growby;
static	int		configfile::ttl;
static	char		*configfile::endofsession;
static	int		configfile::sessiontimeout;
static	char		*configfile::runasuser;
static	char		*configfile::runasgroup;

static	usernode	*configfile::firstuser;
static	usernode	*configfile::currentuser;
static	int		configfile::userlength;

static	connectstringnode	*configfile::firstconnect;
static	connectstringnode	*configfile::currentconnect;
static	int			configfile::connectlength;
#endif

configfile::configfile() {
	port=0;
	unixport="";
	dbase="";
	connections=0;
	maxconnections=0;
	maxqueuelength=0;
	growby=0;
	ttl=0;
	endofsession="";
	runasuser="";
	runasgroup="";
	firstuser=NULL;
	currentuser=NULL;
	firstconnect=NULL;
	currentconnect=NULL;
}

configfile::~configfile() {

	if (dbase[0]) {
		delete[] dbase;
	}

	if (unixport[0]) {
		delete[] unixport;
	}

	if (endofsession[0]) {
		delete[] endofsession;
	}
	if (runasuser[0]) {
		delete[] runasuser;
	}
	if (runasgroup[0]) {
		delete[] runasgroup;
	}

	currentuser=firstuser;
	while (currentuser) {
		firstuser=currentuser->next;
		delete currentuser;
		currentuser=firstuser;
	}

	currentconnect=firstconnect;
	while (currentconnect) {
		firstconnect=currentconnect->next;
		delete currentconnect;
		currentconnect=firstconnect;
	}
}

int	configfile::getPort() {
	return port;
}

char	*configfile::getUnixPort() {
	return unixport;
}

char	*configfile::getDbase() {
	return dbase;
}

int	configfile::getConnections() {
	return connections;
}

int	configfile::getMaxConnections() {
	return maxconnections;
}

int	configfile::getMaxQueueLength() {
	return maxqueuelength;
}

int	configfile::getGrowBy() {
	return growby;
}

int	configfile::getTtl() {
	return ttl;
}

char	*configfile::getEndOfSession() {
	return endofsession;
}

int	configfile::getSessionTimeout() {
	return sessiontimeout;
}

char	*configfile::getRunAsUser() {
	return runasuser;
}

char	*configfile::getRunAsGroup() {
	return runasgroup;
}

usernode	*configfile::getUsers() {
	return firstuser;
}

connectstringnode	*configfile::getConnectStrings() {
	return firstconnect;
}

void	configfile::startElement(void *ctx, const xmlChar *fullname,
					const xmlChar **attributes) {

	// don't do anything if we're already done
	if (done) {
		return;
	}

	// if we haven't found the right instance, then check for it
	if (!foundinstance && 
			!strcmp((char *)fullname,"instance")) {

		// is this the correct instance ?
		if (correctId((char **)attributes)) {

			// if so then save the relevent attributes
			saveInstanceValues((char **)attributes);
			foundinstance=1;
		}

	// if we have found the right instance, add any users we find
	} else if (foundinstance && 
			!strcmp((char *)fullname,"user")) {
		addUser((char **)attributes);

	// if we have found the right instance, add any connections we find
	} else if (foundinstance && 
			!strcmp((char *)fullname,"connection")) {
		addConnection((char **)attributes);
	} 
}

void	configfile::endElement(void *ctx, const xmlChar *fullname) {

	// we're done if we've found the right instance at this point
	if (foundinstance && !strcmp((char *)fullname,"instance")) {
		done=1;
	}
}

int	configfile::correctId(char **attributes) {

	// iterate through the attributes, looking for the id attribute
	int	index=0;
	while (!foundinstance && attributes[index]) {

		// check for the id we're looking for
		if (!strcmp((char *)attributes[index],"id") &&
			!strcmp((char *)attributes[index+1],id)) {
			return 1;
		}
		index=index+2;
	}
	return 0;
}

void	configfile::saveInstanceValues(char **attributes) {

	// iterate through the attributes, looking for the 
	// port, unixport, dbase and connections attributes
	int	index=0;
	while (!foundinstance && attributes[index]) {

		if (!strcmp((char *)attributes[index],"port")) {

			if (attributes[index+1]) {
				port=atoi(attributes[index+1]);
			} else {
				port=0;
			}
		}

		if (!strcmp((char *)attributes[index],"socket")) {

			if (attributes[index+1]) {
				unixport=new 
					char[strlen(attributes[index+1])+1];
				strcpy(unixport,attributes[index+1]);
			} else {
				unixport="";
			}
		}

		if (!unixport && !strcmp((char *)attributes[index],"unixport")) {

			if (attributes[index+1]) {
				unixport=new 
					char[strlen(attributes[index+1])+1];
				strcpy(unixport,attributes[index+1]);
			} else {
				unixport="";
			}
		}

		if (!strcmp((char *)attributes[index],"dbase")) {

			if (attributes[index+1]) {
				dbase=new char[strlen(attributes[index+1])+1];
				strcpy(dbase,attributes[index+1]);
			} else {
				dbase="";
			}
		}
		if (!strcmp((char *)attributes[index],"connections")) {

			if (attributes[index+1]) {
				connections=atoi(attributes[index+1]);
			} else {
				connections=0;
			}
		}
		if (!strcmp((char *)attributes[index],"maxconnections")) {

			if (attributes[index+1]) {
				maxconnections=atoi(attributes[index+1]);
			} else {
				maxconnections=0;
			}
		}
		if (!strcmp((char *)attributes[index],"maxqueuelength")) {

			if (attributes[index+1]) {
				maxqueuelength=atoi(attributes[index+1]);
			} else {
				maxqueuelength=0;
			}
		}
		if (!strcmp((char *)attributes[index],"growby")) {

			if (attributes[index+1]) {
				growby=atoi(attributes[index+1]);
			} else {
				growby=0;
			}
		}
		if (!strcmp((char *)attributes[index],"ttl")) {

			if (attributes[index+1]) {
				ttl=atoi(attributes[index+1]);
			} else {
				ttl=0;
			}
		}
		if (!strcmp((char *)attributes[index],"endofsession")) {

			if (attributes[index+1]) {
				endofsession=new char[
						strlen(attributes[index+1])+1];
				strcpy(endofsession,attributes[index+1]);
			} else {
				endofsession="";
			}
		}
		if (!strcmp((char *)attributes[index],"sessiontimeout")) {

			if (attributes[index+1]) {
				sessiontimeout=atoi(attributes[index+1]);
			} else {
				sessiontimeout=0;
			}
		}
		if (!strcmp((char *)attributes[index],"runasuser")) {

			if (attributes[index+1]) {
				runasuser=new char[
					strlen(attributes[index+1])+1];
				strcpy(runasuser,attributes[index+1]);
			} else {
				runasuser="";
			}
		}
		if (!strcmp((char *)attributes[index],"runasgroup")) {

			if (attributes[index+1]) {
				runasgroup=new char[
					strlen(attributes[index+1])+1];
				strcpy(runasgroup,attributes[index+1]);
			} else {
				runasgroup="";
			}
		}
		index=index+2;
	}
}

void	configfile::addUser(char **attributes) {

	// iterate through the attributes, 
	// looking for the user and password attributes
	int	index=0;
	char	*user;
	char	*password;
	while (attributes[index]) {

		if (!strcmp((char *)attributes[index],"user")) {

			if (attributes[index+1]) {
				user=attributes[index+1];
			} else {
				user="";
			}
		}
		if (!strcmp((char *)attributes[index],"password")) {

			if (attributes[index+1]) {
				password=attributes[index+1];
			} else {
				password="";
			}
		}
		index=index+2;
	}

	// add the user and password to the list
	if (!firstuser) {
		firstuser=new usernode(user,password);
		currentuser=firstuser;
		userlength=1;
	} else {
		currentuser->next=new usernode(user,password);
		currentuser=currentuser->next;
		userlength++;
	}
}

void	configfile::addConnection(char **attributes) {

	// iterate through the attributes, looking for the string attribute
	int	index=0;
	char	*connectionid;
	char	*string;
	int	metric;
	while (attributes[index]) {

		if (!strcmp((char *)attributes[index],"connectionid")) {
			if (attributes[index+1]) {
				connectionid=attributes[index+1];
			} else {
				connectionid="";
			}
		}
		if (!strcmp((char *)attributes[index],"string")) {
			if (attributes[index+1]) {
				string=attributes[index+1];
			} else {
				string="";
			}
		}
		if (!strcmp((char *)attributes[index],"metric")) {
			if (attributes[index+1]) {
				metric=atoi(attributes[index+1]);
			} else {
				metric=0;
			}
		}
		index=index+2;
	}

	// add the string to the list
	if (!firstconnect) {
		firstconnect=new 
			connectstringnode(connectionid,string,metric);
		currentconnect=firstconnect;
		connectlength=1;
	} else {
		currentconnect->next=new 
			connectstringnode(connectionid,string,metric);
		currentconnect=currentconnect->next;
		connectlength++;
	}
}

int	configfile::parse(char *config, char *id) {

	// init some variables
	this->id=id;
	foundinstance=0;
	done=0;

	// set up the callbacks
	xmlSAXHandler	saxHandler={
		NULL, /* internalSubset */
		NULL, /* isStandalone */
		NULL, /* hasInternalSubset */
		NULL, /* hasExternalSubset */
		NULL, /* resolveEntity */
		NULL, /* getEntity */
		NULL, /* entityDecl */
		NULL, /* notationDecl */
		NULL, /* attributeDecl */
		NULL, /* elementDecl */
		NULL, /* unparsedEntityDecl */
		NULL, /* setDocumentLocator */
		NULL, /* startDocument */
		NULL, /* endDocument */
		(void(*)(void *,const xmlChar *,const xmlChar **))
			&configfile::startElement, /* startElement */
		(void(*)(void *,const xmlChar *))
			&configfile::endElement, /* endElement */
		NULL, /* reference */
		NULL, /* characters */
		NULL, /* ignorableWhitespace */
		NULL, /* processingInstruction */
		NULL, /* comment */
		NULL, /* xmlParserWarning */
		NULL, /* xmlParserError */
		NULL, /* xmlParserError */
		NULL, /* getParameterEntity */
		NULL, /* cdataBlock */
	};

	// parse the file
	int	retval=1;
	if (xmlSAXUserParseFile(&saxHandler,NULL,config)<0) {
		cout << "couldn't parse config file: " << config << endl;
		retval=0;
	}

	// if the specified instance wasn't found, warn the user
	if (!done) {
		cout << "Warning: couldn't find instance " << id << "." << endl;
	}

	return retval;
}

usernode::usernode(char *user, char *password) {
	this->user=new char[strlen(user)+1];
	this->password=new char[strlen(password)+1];
	strcpy(this->user,user);
	strcpy(this->password,password);
	next=NULL;
}

usernode::~usernode() {
	delete[] user;
	delete[] password;
}

char	*usernode::getUser() {
	return user;
}

char	*usernode::getPassword() {
	return password;
}

usernode	*usernode::getNext() {
	return next;
}

connectstringnode::connectstringnode(char *connectionid,
					char *string, int metric) {
	this->connectionid=new char[strlen(connectionid)+1];
	strcpy(this->connectionid,connectionid);
	this->string=new char[strlen(string)+1];
	strcpy(this->string,string);
	this->metric=metric;
	next=NULL;
}

connectstringnode::~connectstringnode() {
	delete[] string;
	delete[] connectionid;
}

char	*connectstringnode::getConnectionId() {
	return connectionid;
}

char	*connectstringnode::getString() {
	return string;
}

int	connectstringnode::getMetric() {
	return metric;
}

connectstringnode	*connectstringnode::getNext() {
	return next;
}
