// Copyright (c) 2000  David Muse
// See the file COPYING for more information

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include <config.h>

#include <strstream.h>

#if defined(HAVE_LIBXML)
	#include <libxml/parser.h>
#elif defined(HAVE_GNOME_XML)
	#include <gnome-xml/parser.h>
#endif

class usernode {
	friend class configfile;
	public:
				usernode(char *user, 
						char *password);
				~usernode();
		char		*getUser();
		char		*getPassword();
		usernode	*getNext();
	private:
		char		*user;
		char		*password;
		usernode	*next;
};

class connectstringnode {
	friend class configfile;
	public:
				connectstringnode(char *connectionid,
							char *string, 
							int metric);
				~connectstringnode();
		char			*getConnectionId();
		char			*getString();
		int			getMetric();
		connectstringnode	*getNext();
	private:
		char			*connectionid;
		char			*string;
		int			metric;
		connectstringnode	*next;
};

class configfile {
	public:
			configfile();
			~configfile();
			int	parse(char *config, char *id);
		static	int	getPort();
		static	char	*getUnixPort();
		static	char	*getDbase();
		static	int	getConnections();
		static	int	getMaxConnections();
		static	int	getMaxQueueLength();
		static	int	getGrowBy();
		static	int	getTtl();
		static	char	*getEndOfSession();
		static	int	getSessionTimeout();
		static	char	*getRunAsUser();
		static	char	*getRunAsGroup();

		static	usernode		*getUsers();
		static	connectstringnode	*getConnectStrings();
	private:
		static	char		*id;
		static	strstream	connectstring;
		static	int		foundinstance;
		static	int		done;

		static	void	startElement(void *ctx, const xmlChar *fullname,
						const xmlChar **attributes);
		static	void	endElement(void *ctx, const xmlChar *fullname);

		static	int	correctId(char **attributes);
		static	void	saveInstanceValues(char **attributes);
		static	void	addUser(char **attributes);
		static	void	addConnection(char **attributes);

		static	int		port;
		static	char		*unixport;
		static	char		*dbase;
		static	int		connections;
		static	int		maxconnections;
		static	int		maxqueuelength;
		static	int		growby;
		static	int		ttl;
		static	char		*endofsession;
		static	int		sessiontimeout;
		static	char		*runasuser;
		static	char		*runasgroup;

		static	usernode	*firstuser;
		static	usernode	*currentuser;
		static	int		userlength;

		static	connectstringnode	*firstconnect;
		static	connectstringnode	*currentconnect;
		static	int			connectlength;
};

#endif
