// Copyright (c) 1999  David Muse
// See the file COPYING for more information

#ifndef CONNECTION_H
#define CONNECTION_H

#include <config.h>
#include <defaults.h>
#include <signalhandler.h>
#include <daemon.h>
#include <server.h>
#include <semaphoreset.h>
#include <sharedmemory.h>

#include <strstream.h>
#include <fstream.h>

#define BINDVARLENGTH 30
#define BINDVALUELENGTH 4000
#define MAXQUERYSIZE 32768
#define MAXVAR 256

enum bindtype {
	STRING,
	LONG,
	DOUBLE
};

class bindvar {
	public:
		char		variable[BINDVARLENGTH+2];
		short		variablesize;
		union {
			char	stringval[BINDVALUELENGTH+1];
			long	longval;
			struct	{
				double	value;
				short	precision;
				short	scale;
			} doubleval;
		} value;
		short		valuesize;
		bindtype	type;
		short		isnull;
};

class connection : public daemon, public server {
	public:
			connection();
		virtual	~connection();
			void	initConnection(int argc, char **argv);
			void	listen();
			void	closeConnection();
	protected:

		// interface definition
		virtual	int	getNumberOfConnectStringVars()=0;
		virtual	void	handleConnectString()=0;
		virtual	int	logIn()=0;
		virtual	void	logOut()=0;
		virtual	int	prepareQuery(char *query, long length);
		virtual	int	inputBindValue(char *variable, 
						short variablesize,
						char *value, 
						short valuesize,
						short *isnull);
		virtual	int	inputBindValue(char *variable, 
						short variablesize,
						long *value);
		virtual	int	inputBindValue(char *variable, 
						short variablesize,
						double *value,
						short precision,
						short scale);
		virtual	int	outputBindValue(char *variable, 
						short variablesize,
						char *value,
						short valuesize,
						short *isnull);
		virtual	short	nonNullBindValue();
		virtual	short	nullBindValue();
		virtual int	bindValueIsNull(short isnull);
		virtual	int	executeQuery(char *query, long length)=0;
		virtual	void	returnHeader()=0;
		virtual	int	noRowsToReturn()=0;
		virtual	int	fetchRow()=0;
		virtual	void	returnRow()=0;
		virtual	void	cleanUpData()=0;
		virtual	char	*getErrorMessage(int *liveconnection)=0;
		virtual	int	ping()=0;
		virtual	char	*identify()=0;

		// transactional methods
		virtual	int	queryIsNotSelect();
		virtual	int	queryIsCommitOrRollback();
		virtual	void	commit();
		virtual	void	rollback();
		virtual int	isTransactional();


			void	parseConnectString(char *connectstring);
			void	reLogIn();
			void	announceAvailability();
			void	initSession();
			void	clientSession();
			int	getQuery();
			int	getInputBinds();
			int	getOutputBinds();
			int	processQuery();
			void	fakeBinds();
			int	handleBinds();
			void	commitOrRollback();
			int	pingOrIdentify();
			int	handleError();
			int	returnError();
			void	returnResultSet();
			void	returnOutputBindValues();
			int	returnData();

			void	sendRowCounts(long actual, long affected);
			void	sendColumnDefinition(char *name, short namelen, 
							short type, long size);
			long	rowsToFetch();
			long	rowsToSkip();
		virtual	int	skipRows(int rows);
			void	sendField(char *data, long size);
			void	sendNullField();
			void	startSendingLong();
			void	sendLongSegment(char *data, long size);
			void	endSendingLong();
		
			char	*connectStringValue(char *variable);

			int	setEnv(char *var, char *string);

			void	markDatabaseUnavailable();
			void	markDatabaseAvailable();
			void	waitForAvailableDatabase();

			void	endOfSession();

			char	*getUnixSocket();

			void	incrementConnectionCount();
			void	decrementConnectionCount();
			void	decrementSessionCount();

			void	blockSignals();
			void	openSockets();

		strstream	*unixsocket;
		strstream	*sockseqname;

		char		*connectstring;
		int		connectstringcount;
		strstream	**connectstringvar;
		strstream	**connectstringval;

		int		ttl;

		char		querybuffer[MAXQUERYSIZE+1];
		long		querylength;
		bindvar		inbindvars[MAXVAR];
		short		inbindcount;
		bindvar		outbindvars[MAXVAR];
		short		outbindcount;

		strstream	*idfilename;

		strstream	*updown;

		semaphoreset	*semset;

		sharedmemory	*idmemory;

		int		init;

		int		commitorrollback;
		int		eoscommit;
		int		sessiontimeout;
		int		accepttimeout;
		int		suspendsession;
		int		suspendresultset;
		long		lastrow;

		int		dynamicscaling;
};

#endif
