// Copyright (c) 1999-2000  David Muse
// See the file COPYING for more information

#include <defaults.h>
#include <commandline.h>
#include <configfile.h>

#include <iostream.h>
#include <strstream.h>

#include <math.h>
#include <string.h>
#include <config.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

int	main(int argc, char **argv) {

	#include <version.h>

	// declare the command buffer
	strstream	*command;

	// get the command line args
	commandline	*cmdl=new commandline(argc,argv);

	char	*id=cmdl->value("-id");
	if (!id[0]) {
		id=DEFAULT_ID;
		cerr << "Warning! using default id." << endl;
	}
	char	*config=cmdl->value("-config");
	if (!config[0]) {
		config=DEFAULT_CONFIG_FILE;
	}

	// parse the config file
	configfile	*cfgfile=new configfile();
	if (cfgfile->parse(config,id)) {

		// get number of connections
		int	cons=cfgfile->getConnections();
		if (cons==0) {
			cons=1;
		}
		if (cons>MAX_CONNECTIONS) {
			cons=MAX_CONNECTIONS;
		}

		// get database type
		char	*dbase=cfgfile->getDbase();
		if (!(dbase && dbase[0])) {
			dbase=DEFAULT_DBASE;
			cerr << "Warning! using default database type." << endl;
		}

		// get connect strings
		connectstringnode	*connectionlist=
						cfgfile->getConnectStrings();

		// get the total of all metrics
		connectstringnode	*currentnode=connectionlist;
		int			metrictotal=0;
		int			connectioncount=0;
		while(currentnode) {
			metrictotal=metrictotal+currentnode->getMetric();
			currentnode=currentnode->getNext();
			connectioncount++;
		}

		// initialize some useful variables
		int	startup=0;
		int	totalstarted=0;
		int	done=0;

		// if the metric total was 0 then start no connections
		if (metrictotal==0) {
			done=1;
		}

		// if no connections were defined, start 1 default one
		if (!connectioncount) {
			command=new strstream();
			*command << "sqlr-connection-" << dbase;
			*command << " -id " << id;
			*command << " -config " << config;
			*command << ends;
			cout << "  " << command->str() << endl;
			system(command->str());
			delete command;
			sleep(1);
		}

		// start the connections
		currentnode=connectionlist;
		int	metric=0;
		while (!done) {
	
			// scale the number of each connection to start by 
			// each connection's metric vs the total number of 
			// connections to start up
			metric=currentnode->getMetric();
			if (metric>0) {
				startup=(int)ceil(
					((double)(metric*cons))/
					((double)metrictotal));
			} else {
				startup=0;
			}

			// keep from starting too many connections
			if (totalstarted+startup>cons) {
				startup=cons-totalstarted;
				done=1;
			}

			cout << endl << "Starting " << startup;
			cout << " connections to ";
			cout << currentnode->getConnectionId() << " :" << endl;

			// fire them up
			for (int i=0; i<startup; i++) {
	
				command=new strstream();
				*command << "sqlr-connection-" << dbase;
				*command << " -id " << id;
				*command << " -config " << config;
				*command << " -connectionid ";
				*command << currentnode->getConnectionId();
				*command << ends;
				cout << "  " << command->str() << endl;
				system(command->str());
				delete command;
		
				sleep(1);
			}

			// have we started enough connections?
			totalstarted=totalstarted+startup;
			if (totalstarted==cons) {
				done=1;
			}

			// next...
			if (currentnode) {
				currentnode=currentnode->getNext();
			}
		}
	
		// start the listener
		cout << endl << "Starting listener:" << endl;
	
		command=new strstream();
		*command << "sqlr-listener ";
		*command << " -id " << id;
		*command << " -config " << config << ends;
		cout << "  " << command->str() << endl;
		system(command->str());
		delete command;

		// start the scaler
		if (cfgfile->getMaxQueueLength()>=0 &&
			cfgfile->getMaxConnections()>
				cfgfile->getConnections() &&
			cfgfile->getGrowBy()>0 &&
			cfgfile->getTtl()>0) {

			cout << endl << "Starting scaler:" << endl;
	
			command=new strstream();
			*command << "sqlr-scaler ";
			*command << " -id " << id;
			*command << " -config " << config << ends;
			cout << "  " << command->str() << endl;
			system(command->str());
			delete command;
		}
		
		// start the cache manager (if it's not already running)
		command=new strstream();
		*command << "ps -efa | grep sqlr-cachemanager | grep -v grep";
		*command << ends;
	
		FILE	*cmd=popen(command->str(),"r");
		delete command;
	
		strstream	*contents=new strstream();
		char		character;
		while (!feof(cmd) && (character=fgetc(cmd))>-1) {
			*contents << character;
		}
		*contents << ends;
	
		if (contents->rdbuf()->pcount()-1==0) {
	
			cout << endl << "Starting cache manager:" << endl;
	
			command=new strstream();
			*command << "sqlr-cachemanager" << ends;
			cout << "  " << command->str() << endl;
			system(command->str());
			delete command;
	
		} else {
			cout << endl << "cache manager already running.";
			cout << endl;
		}
		pclose(cmd);
	
		delete contents;
	}

	// many thanks...
	cout << endl << endl;
	cout << "Thanks to MP3.com for sponsoring: " << endl;
	cout << "	Clustered/Replicated database support." << endl;
	cout << "	Perl API." << endl;
	

	delete cfgfile;
	delete cmdl;

}
