#! /usr/bin/env perl

# Copyright (c) 2000  David Muse
# See the file COPYING for more information.

use Firstworks::SQLRClient;

# usage...
if ($#ARGV+1<6) {
	print("usage: perl.pl host port socket user password query\n");
	exit;
}


# instantiation
print "INSTANTIATION\n";
$s=Firstworks::SQLRClient->new($ARGV[0],$ARGV[1],$ARGV[2],$ARGV[3],$ARGV[4],0,1);
print "\n";
print "\n";


# debugOn
print "DEBUG ON\n";
$s->debugOn();
print("getDebug(): ", $s->getDebug(), "\n");
print "\n";
print "\n";


# query functions
print "QUERY FUNCTIONS\n";
$s->sendQuery($ARGV[5]);
print "\n";
print "\n";


# prepare/substutute/bind/execute functions
print "BIND FUNCTIONS\n";
$s->prepareQuery("begin  \$(result):=\$(term1)+\$(term2)+:term3+:term4+:term5; end;");
$s->substitution("result",":result");
$s->substitution("term1",400);
$s->substitution("term2",200.0);
$s->inputBind("term3","100");
$s->inputBind("term4",50);
$s->inputBind("term5",25.0);
$s->inputBind("junkterm","junkvalue");
$s->defineOutputBind("result",5);
$s->defineOutputBind("junkresult",5);
$s->validateBinds();
$s->executeQuery();
$result=$s->getOutputBind("result");
$junkresult=$s->getOutputBind("junkresult");
print "result is ", $result, "\n";
print "junkresult is ", $junkresult, "\n";
print "\n";
print "\n";


# cached query functions
print "CACHED QUERY FUNCTIONS\n";
$s->cacheOn();
$s->setCacheTtl(1000);
$s->sendQuery($ARGV[5]);
$filename=$s->getCacheFileName();
$s->cacheOff();
$s->openCachedResultSet($filename);
print("rowCount() is ", $s->rowCount(), "\n");
print "\n";


# result set functions
print "RESULT SET FUNCTIONS\n";
$s->sendQuery($ARGV[5]);
print("rowCount()    : ", $s->rowCount(), "\n");
print("colCount()    : ", $s->colCount(), "\n");
print("totalRows()   : ", $s->totalRows(), "\n");
print("affectedRows(): ", $s->affectedRows(), "\n");
print "\n";
print "\n";


# column functions
print "COLUMN FUNCTIONS\n";
for ($col=0; $col<$s->colCount(); $col++) {
	print('"', $s->getColumnName($col), '","', $s->getColumnType($col), '",', $s->getColumnLength($col), ',', $s->getLongest($col), "\n");
}
@cols=$s->getColumnNames();
for ($col=0; $col<$s->colCount(); $col++) {
	print($cols[$col],"\n");
}
print "\n";
print "\n";


# field functions
print "FIELD FUNCTIONS\n";
for ($row=0; $row<$s->rowCount(); $row++) {
	$colcount=$s->colCount();
	for ($col=0; $col<$s->colCount(); $col++) {
		print('"', $s->getField($row, $col), '"');
		if ($col < $colcount-1) {
			print(',');
		}
	}
	print("\n");
}
print "\n";
print "\n";


# row functions
print "ROW FUNCTIONS\n";
for ($row=0; $row<$s->rowCount(); $row++) {
	print("Row as array:\n");
	@row=$s->getRow($row);
	print(@row);
	print("\n");
	print("Row as hash:\n");
	%row=$s->getRowHash($row);
	print(%row);
	print("\n");
}
print "\n";
print "\n";


# null functions
print "NULL FUNCTIONS\n";
$s->getNullsAsUndefined();
$s->sendQuery("select NULL \"COL1\", NULL \"COL2\", NULL  \"COL3\" from dual");
for ($row=0; $row<$s->rowCount(); $row++) {
	print("Row as array:\n");
	@row=$s->getRow($row);
	for ($col=0; $col<$s->colCount(); $col++) {
		if (defined($row[$col])) {
			printf("%d is defined\n",$col);
		} else {
			printf("%d is undefined\n",$col);
		}
	}
	print("Row as hash:\n");
	%row=$s->getRowHash($row);
	for ($col=0; $col<$s->colCount(); $col++) {
		if (defined($row[$s->getColumnName($col)])) {
			printf("%s is defined\n",$s->getColumnName($col));
		} else {
			printf("%s is undefined\n",$s->getColumnName($col));
		}
	}
	print("\n");
}
$s->getNullsAsEmptyStrings();
print "\n";
print "\n";


# result set buffer functions
print "RESULT SET BUFFER FUNCTIONS\n";
$s->setResultSetBufferSize(3);
print("getResultSetBufferSize(): ", $s->getResultSetBufferSize(), "\n");
$s->sendQuery($ARGV[5]);
$field=$s->getField(0,0);
print("getField(0,0)  : ", $field, "\n");
$field=$s->getField(100,0);
print("getField(100,0): ", $field, "\n");
print "\n";
print "\n";


# cached result set with offset functions
print "CACHED RESULT SET WITH OFFSET FUNCTIONS\n";
$s->setResultSetBufferSize(0);
$s->cacheOn();
$s->setCacheTtl(1000);
$s->sendQuery($ARGV[5]);
$s->cacheOff();
$filename=$s->getCacheFileName();
$s->setResultSetBufferSize(3);
$s->openCachedResultSet($filename);
$field=$s->getField(5,0);
print("getField(5,0)  : ", $s->getField(5,0), "\n");
print("rowCount()     : ", $s->rowCount(), "\n");
$offset=$s->getCacheFileOffset();
$row=$s->rowCount();
print("offset: ", $offset, "\n");
print("row   : ", $row, "\n");
$s->openCachedResultSetWithOffset($filename,$offset,$row);
print("getField(8,0)  : ", $s->getField(8,0), "\n");
print("rowCount()     : ", $s->rowCount(), "\n");
print "\n";
print "\n";


# suspended session
print "SUSPENDED SESSION\n";
$s->setResultSetBufferSize(3);
$s->sendQuery($ARGV[5]);
$port=$s->getConnectionPort();
$socket=$s->getConnectionSocket();
$s->suspendSession();
$s->resumeSession($port,$socket);
print("firstRowIndex  : ", $s->firstRowIndex(), "\n");
$field=$s->getField(0,0);
print("getField(0,0)  : ", $field, "\n");
$field=$s->getField(5,0);
print("getField(5,0)  : ", $field, "\n");
$field=$s->getField(6,0);
print("getField(6,0)  : ", $field, "\n");
$field=$s->getField(100,0);
print("getField(100,0)  : ", $field, "\n");
print "\n";
print "\n";



# suspended cached session
print "SUSPENDED CACHED SESSION\n";
$s->setResultSetBufferSize(3);
$s->cacheOn();
$s->setCacheTtl(1000);
$s->sendQuery($ARGV[5]);
$s->cacheOff();
$filename=$s->getCacheFileName();
$port=$s->getConnectionPort();
$socket=$s->getConnectionSocket();
$s->suspendSession();
$s->resumeCachedSession($port,$socket,$filename);
$s->endSession();
$s->setResultSetBufferSize(0);
$s->openCachedResultSet($filename);


# ping and identify
print "PING AND IDENTIFY\n";
print "identify: ", $s->identify(), "\n";
print "ping: ", $s->ping(), "\n";


# debugOff
print "DEBUG OFF\n";
$s->debugOff();
print("getDebug(): ", $s->getDebug(), "\n");
print "\n";
print "\n";
