#! /usr/bin/env python

# Copyright (c) 2000  David Muse
# See the file COPYING for more information.

from SQLRelay import PySQLRClient
import sys
import string


def main():

	# usage...
	if len(sys.argv) < 6:
		print "usage: pythontest.py host port socket user password query"
		sys.exit(0)


	# instantiation
	print "INSTANTIATION"
	if len(sys.argv[3])==0:
		s = PySQLRClient.sqlrclient(
				(sys.argv[1], string.atoi(sys.argv[2])),
                       			sys.argv[4],sys.argv[5])
	else:
		s = PySQLRClient.sqlrclient(
				sys.argv[3],sys.argv[4],sys.argv[5])
	print
	print


	# debugOn
	print "DEBUG ON"
	s.debugOn()
	print "getDebug(): ", s.getDebug()
	print
	print


	# query functions
	print "QUERY FUNCTIONS"
	s.sendQuery(sys.argv[6])
	print
	print


	# prepare/substutute/bind/execute functions
	print "BIND FUNCTIONS\n";
	s.prepareQuery("begin  $(result):=$(term1)+$(term2)+:term3+:term4+:term5; end;");
	s.substitution("result",":result");
	s.substitution("term1",400.0);
	s.substitution("term2",200);
	s.inputBind("term3","100");
	s.inputBind("term4",50);
	s.inputBind("term5",25.0);
	s.inputBind("junkterm","junkvalue");
	s.defineOutputBind("result",5);
	s.validateBinds();
	s.executeQuery();
	result=s.getOutputBind("result");
	junkresult=s.getOutputBind("junkresult");
	print "result is ", result;
	print "junkresult is ", junkresult;
	print "\n";
	print "\n";


	# cached query functions
	print "CACHED QUERY FUNCTIONS"
	s.cacheOn()
	s.setCacheTtl(1000)
	s.sendQuery(sys.argv[6])
	print "rowCount() is ", s.rowCount()
	print
	filename=s.getCacheFileName()
	s.cacheOff()
	s.openCachedResultSet(filename)
	print "rowCount() is ", s.rowCount()
	print
	print


	# result set functions
	print "RESULT SET FUNCTIONS"
	s.sendQuery(sys.argv[6])
	print 'rowCount()    : ', s.rowCount()
	print 'colCount()    : ', s.colCount()
	print 'totalRows()   : ', s.totalRows()
	print 'affectedRows(): ', s.affectedRows()
	print
	print


	# column functions
	print "COLUMN FUNCTIONS"
	for col in range(s.colCount()):
		print '"' + s.getColumnName(col) + '","' + s.getColumnType(col) + '",', s.getColumnLength(col), ',', s.getLongest(col)
	print
	print


	# field functions
	print "FIELD FUNCTIONS"
	for row in range(s.rowCount()):
		colcount=s.colCount()
		for col in range(s.colCount()):
			print '"' + string.strip(s.getField(row, col)) + '"',
			if col < colcount-1:
				print ',',
		print 
	print
	print


	# null functions
	print "NULL FUNCTIONS"
	s.getNullsAsNone()
	s.sendQuery("select NULL \"COL1\", NULL \"COL2\", NULL \"COL3\" from dual")
	for row in range(0,s.rowCount()):
		print "Row as array:"
		rowarray=s.getRow(row)
		print rowarray
		print
		print "Row as dictionary:"
		rowdict=s.getRowDictionary(row)
		print rowdict
		print
	s.getNullsAsEmptyStrings()
	print
	print


	# result set functions
	print "RESULT SET BUFFER FUNCTIONS"
	s.setResultSetBufferSize(3)
	print 'getResultSetBufferSize(): ', s.getResultSetBufferSize()
	s.sendQuery(sys.argv[6])
	field=s.getField(0,0)
	print "getField(0,0)  : ", field
	field=s.getField(100,0)
	print "getField(100,0): ", field
	print
	print


	# cached result set with offset functions
	print "CACHED RESULT SET WITH OFFSET FUNCTIONS"
	s.setResultSetBufferSize(0)
	s.cacheOn()
	s.setCacheTtl(1000)
	s.sendQuery(sys.argv[6])
	filename=s.getCacheFileName()
	s.cacheOff()
	s.setResultSetBufferSize(3)
	s.openCachedResultSet(filename)
	field=s.getField(5,0)
	print "getField(5,0)  : ", s.getField(5,0)
	print "rowCount()     : ", s.rowCount()
	offset=s.getCacheFileOffset()
	row=s.rowCount()
	print "offset: ", offset
	print "row   : ", row
	s.openCachedResultSetWithOffset(filename,offset,row)
	print "getField(8,0)  : ", s.getField(8,0)
	print "rowCount()     : ", s.rowCount()
	print
	print


	# suspended session
	print "SUSPENDED SESSION"
	s.setResultSetBufferSize(3)
	s.sendQuery(sys.argv[6])
	port=s.getConnectionPort()
	socket=s.getConnectionSocket()
	s.suspendSession()
	s.resumeSession(port,socket)
	print "firstRowIndex  : ", s.firstRowIndex()
	field=s.getField(0,0)
	print "getField(0,0)  : ", field
	field=s.getField(5,0)
	print "getField(5,0)  : ", field
	field=s.getField(6,0)
	print "getField(6,0)  : ", field
	field=s.getField(100,0)
	print "getField(100,0)  : ", field
	print
	print


	# suspended cached session
	print "SUSPENDED CACHED SESSION"
	s.setResultSetBufferSize(3)
	s.cacheOn()
	s.setCacheTtl(1000)
	s.sendQuery(sys.argv[6])
	filename=s.getCacheFileName()
	s.cacheOff()
	port=s.getConnectionPort()
	socket=s.getConnectionSocket()
	s.suspendSession()
	s.resumeCachedSession(port,socket,filename)
	s.endSession()
	s.setResultSetBufferSize(0)
	s.openCachedResultSet(filename)
	print
	print


	# python specific api calls
	print "PYTHON SPECIFIC CALLS"
	s.sendQuery(sys.argv[6])
	cols=s.getColumnNames()
	rows=s.getRowRange(0,5)
	print "Columns:"
	print cols
	print "Rows 0-5:"
	print rows
	print "Row Dictionaries:"
	for i in range(0,s.rowCount()):
		rowdict=s.getRowDictionary(i)
		print rowdict
	print
	print


	# ping and identify
	print "PING AND IDENTIFY";
	print "identify: ", s.identify()
	print "ping: ", s.ping()


	# debugOff
	print "DEBUG OFF"
	s.debugOff()
	print "getDebug(): ", s.getDebug()
	print
	print
                    

if __name__ == '__main__':
	main()
